/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manages the about dialog
//-----------------------------------------------------------------------------

#include "about.h"

CAbout::CAbout()
{
    this->hWndDialog=NULL;
    this->hFontAboutTitle=NULL;
    this->pSysLinkAbout=NULL;
    this->pSyslinkText=NULL;
    this->hFontAboutLink=NULL;
}
CAbout::~CAbout()
{
    if (this->pSyslinkText)
        free(this->pSyslinkText);
}


//-----------------------------------------------------------------------------
// Name: Show
// Object: display the about dialog
// Parameters :
//     in  : HINSTANCE hInstance : instance containing dialog resource
//           HWND hWndDialog : parent window dialog handle
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CAbout::Show(HINSTANCE hInstance,HWND hWndDialog,TCHAR* pSyslinkText)
{
    CAbout About;


    if (pSyslinkText)
        About.pSyslinkText=_tcsdup(pSyslinkText);
    else
        About.pSyslinkText=_tcsdup(ABOUT_SYSLINK_TEXT);

    TCHAR AppFileName[MAX_PATH];
    TCHAR PrettyVersion[MAX_PATH];
    
    CStdFileOperations::GetAppName(AppFileName,MAX_PATH);
    CVersion VersionInfo;
    if (VersionInfo.Read(AppFileName))
    {
        CVersion::GetPrettyVersion(VersionInfo.ProductVersion,3,PrettyVersion,MAX_PATH);
        _sntprintf(About.NameAndVersion,MAX_PATH,_T("%s v%s"),
                    VersionInfo.ProductName,
                    PrettyVersion
                    );
    }
    else
        *About.NameAndVersion=0;

    
    ::DialogBoxParam(hInstance, (LPCTSTR)IDD_DIALOG_ABOUT, hWndDialog, (DLGPROC)CAbout::WndProc,(LPARAM)&About);

}

//-----------------------------------------------------------------------------
// Name: AboutWndProc
// Object: about dialog window proc
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CALLBACK CAbout::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch (uMsg)
    {
    case WM_INITDIALOG:
        {
            CAbout* pAbout=(CAbout*)lParam;
            pAbout->hWndDialog=hWnd;
            ::SetWindowLongPtr(pAbout->hWndDialog,GWLP_USERDATA,(LONG_PTR)lParam);
            // load dlg icons
            CDialogHelper::SetIcon(pAbout->hWndDialog,IDI_ICON_ABOUT);

            // set Font for Title
            pAbout->hFontAboutTitle=::CreateFont(ABOUT_TITLE_FONT_HEIGHT,0,0,0,FW_BOLD,0,0,0,
                            ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH,
                            _T("MS Shell Dlg"));
            ::SendMessage(GetDlgItem(hWnd,IDC_STATIC_ABOUT_TITLE),WM_SETFONT,(WPARAM)pAbout->hFontAboutTitle,0);

            // set title (name and version)
            ::SetDlgItemText(pAbout->hWndDialog,IDC_STATIC_ABOUT_TITLE,pAbout->NameAndVersion);

            // show syslink
            pAbout->pSysLinkAbout=new CSysLink(hWnd,ID_ABOUT_SYSLINK,
                                                pAbout->pSyslinkText,
                                                ABOUT_SYSLINK_XPOS,
                                                ABOUT_SYSLINK_YPOS,
                                                ABOUT_SYSLINK_WIDTH,
                                                ABOUT_SYSLINK_HEIGHT);
            // set Font for syslink
            pAbout->hFontAboutLink=::CreateFont(ABOUT_LINK_FONT_HEIGHT,0,0,0,FW_NORMAL,0,0,0,
                                              ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH,
                                              _T("MS Shell Dlg"));
            ::SendMessage(pAbout->pSysLinkAbout->GetControlHandle(),WM_SETFONT,(WPARAM)pAbout->hFontAboutLink,0);

            CDialogHelper::SetTransparency(hWnd,ABOUT_OPACITY);
        }

        break;
    case WM_CLOSE:
        {
            CAbout* pAbout=CAbout::GetAssociatedObject(hWnd);
            if (!pAbout)
                break;
        
            pAbout->OnExit();
        }
        break;
    case WM_LBUTTONDOWN:
        break;
    case WM_LBUTTONUP:
    case WM_KILLFOCUS:
        break;
    case WM_MOUSEMOVE:
        break;

    case WM_COMMAND:
        {
            CAbout* pAbout=CAbout::GetAssociatedObject(hWnd);
            if (!pAbout)
                break;

            switch (LOWORD(wParam))
            {
            case IDOK:
                pAbout->OnExit();
                break;
            }
        }
        break;
    case WM_NOTIFY:
        {
            CAbout* pAbout=CAbout::GetAssociatedObject(hWnd);
            if (!pAbout)
                break;

            WCHAR IDName[48];
            if (pAbout->pSysLinkAbout)
            {
                if (pAbout->pSysLinkAbout->Notify(lParam,(LPWSTR)&IDName))
                {
                    if (wcscmp(IDName,L"IdHomePage")==0)
                    {
                        pAbout->pSysLinkAbout->OpenLink(ABOUT_SYSLINK_PROJECT_LINK);
                    }
                    else if (wcscmp(IDName,L"IdUserDoc")==0)
                    {
                        pAbout->pSysLinkAbout->OpenLink(ABOUT_SYSLINK_PROJECT_USER_DOC_LINK);
                    }
                    else // IdMailTo
                    {
                        pAbout->pSysLinkAbout->OpenLink(ABOUT_SYSLINK_EMAIL_LINK);
                    }
                }
            }
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}
//-----------------------------------------------------------------------------
// Name: GetAssociatedObject
// Object: Get object associated to window handle
// Parameters :
//     in  : HWND hWndDialog : handle of the window
//     out :
//     return : associated object if found, NULL if not found
//-----------------------------------------------------------------------------
CAbout* CAbout::GetAssociatedObject(HWND hWndDialog)
{
    return (CAbout*)::GetWindowLongPtr(hWndDialog,GWLP_USERDATA);
}
//-----------------------------------------------------------------------------
// Name: AboutDlgExit
// Object: close the about dialog
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CAbout::OnExit()
{
    if (this->hWndDialog)
    {
        // no more required on vista and upper
        // CDialogHelper::Fading(this->hWndDialog,ABOUT_OPACITY,0,ABOUT_FADING_STEP);
        ::DeleteFont(this->hFontAboutTitle);
        ::DeleteFont(this->hFontAboutLink);
        ::EndDialog(this->hWndDialog,0);
        this->hWndDialog=NULL;
    }
    if (this->pSysLinkAbout)
    {
        delete this->pSysLinkAbout;
        this->pSysLinkAbout=NULL;
    }
}