/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "remotectrlcontentsaver.h"

//-----------------------------------------------------------------------------
// Name: SendMessageThreadProc
// Object: proc that will be injected in remote process (just will call SendMessageW)
// Parameters :
//     in  : CRemoteCtrlContentSaver::PDATASENDMESSAGE lpParameter, containing
//              SendMessageW address and SendMessageW parameters
//     out : 
//     return : 
//-----------------------------------------------------------------------------
static DWORD WINAPI SendMessageThreadProc (CRemoteCtrlContentSaver::PDATASENDMESSAGE lpParameter)
{
    lpParameter->Lresult=lpParameter->pSendMessage(lpParameter->Hwnd,lpParameter->Msg,lpParameter->Wparam,lpParameter->Lparam);
 	return 0;
}
// This function marks the memory address after ThreadProc.
static DWORD WINAPI  SendMessageAfterThreadProc (void) {return 0;}


void CRemoteCtrlContentSaver::CommonConstructor()
{
    this->hProcess=NULL;
    this->pSelectWindow=NULL;
    this->pRemoteTreeViewItem=NULL;
    this->pRemoteItemText=NULL;
    this->TextBufferSize=REMOTECTRLCONTENTSAVER_DEFAULT_TEXT_BUFFER_SIZE;

}

//-----------------------------------------------------------------------------
// Name: CRemoteCtrlContentSaver
// Object: constructor. used to avoid user message
//         CRemoteCtrlContentSaver(TRUE) is equivalent to CRemoteCtrlContentSaver((HWND)NULL))
// Parameters :
//     in  : BOOL bShowMessages
//     out : 
//     return : 
//-----------------------------------------------------------------------------
CRemoteCtrlContentSaver::CRemoteCtrlContentSaver(BOOL bShowMessages)
{
    this->bShowMessages=bShowMessages;
    this->HwndDialog=NULL;
    this->CommonConstructor();
}

//-----------------------------------------------------------------------------
// Name: CRemoteCtrlContentSaver
// Object: constructor
// Parameters :
//     in  : HWND AppDialogHandle : owner dialog, can be NULL (used only for messageboxes)
//     out : 
//     return : 
//-----------------------------------------------------------------------------
CRemoteCtrlContentSaver::CRemoteCtrlContentSaver(HWND AppDialogHandle)
{
    this->bShowMessages=TRUE;
    this->HwndDialog=AppDialogHandle;
    this->CommonConstructor();
}

//-----------------------------------------------------------------------------
// Name: ~CRemoteCtrlContentSaver
// Object: destructor
// Parameters :
//     in  : 
//     out : 
//     return : 
//-----------------------------------------------------------------------------
CRemoteCtrlContentSaver::~CRemoteCtrlContentSaver(void)
{
}

//-----------------------------------------------------------------------------
// Name: ShowError
// Object: show message box with error message 
// Parameters :
//     in  : TCHAR* psz : error message
//     out : 
//     return : 
//-----------------------------------------------------------------------------
void CRemoteCtrlContentSaver::ShowError(TCHAR* psz)
{
    if (this->bShowMessages)
        MessageBox(this->HwndDialog,psz,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
}

//-----------------------------------------------------------------------------
// Name: UpdateControl
// Object: Update the pSelectWindow of the object and return the type of the control
//          For ComboboxEx, pSelectWindow will be updated 
// Parameters :
//     in  : 
//     out : 
//     return : SUPPORTED_CTRL_TYPE
//-----------------------------------------------------------------------------
CRemoteCtrlContentSaver::REMOTECTRLCONTENTSAVER_SUPPORTED_CTRL_TYPES CRemoteCtrlContentSaver::UpdateControl(CSelectWindow* pSelectWindow)
{
    TCHAR* pszClassNameUpper;
    
    this->pSelectWindow=pSelectWindow;
    this->ControlType=SUPPORTED_CTRL_UNSUPPORTED;

    if (!this->pSelectWindow)
        return this->ControlType;

    /////////////////////////
    // find control type
    /////////////////////////
    pszClassNameUpper=_tcsdup(this->pSelectWindow->WindowClassName);
    _tcsupr(pszClassNameUpper);
    
    if ((_tcsstr(pszClassNameUpper,_T("SYSLISTVIEW32"))!=0)
        ||(_tcsstr(pszClassNameUpper,_T("TLISTVIEW"))!=0))
    {
        this->ControlType=SUPPORTED_CTRL_LISTVIEW;
    }
    else if (_tcsstr(pszClassNameUpper,_T("COMBOBOX"))!=0)
    {
        this->ControlType=SUPPORTED_CTRL_COMBOBOX;
        // check if combobox is a Combobox Ex (as selected window can only be the combobox)
        free(pszClassNameUpper);
        pszClassNameUpper=_tcsdup(this->pSelectWindow->ParentWindowClassName);
        _tcsupr(pszClassNameUpper);
        if (_tcsstr(pszClassNameUpper, _T("COMBOBOXEX32"))!=0)
            this->ControlType=SUPPORTED_CTRL_COMBOBOXEX32;
    }
    else if (_tcsstr(pszClassNameUpper,_T("LISTBOX"))!=0)
    {
        this->ControlType=SUPPORTED_CTRL_LISTBOX;
    }
    else if (_tcsstr(pszClassNameUpper,_T("SYSHEADER32"))!=0)
    {
        this->ControlType=SUPPORTED_CTRL_HEADERS;
    }
    else if (_tcsstr(pszClassNameUpper,_T("SYSHEADER32"))!=0)
    {
        this->ControlType=SUPPORTED_CTRL_HEADERS;
    }
    else if ((_tcsstr(pszClassNameUpper,_T("SYSTREEVIEW32"))!=0)
             ||(_tcsstr(pszClassNameUpper,_T("TTREEVIEW"))!=0)
             )
    {
        this->ControlType=SUPPORTED_CTRL_TREEVIEW;
    }
    else if ((_tcsstr(pszClassNameUpper,_T("EDIT"))!=0)
        ||(_tcsstr(pszClassNameUpper,_T("TEXTBOX"))!=0)
        ||(_tcsstr(pszClassNameUpper,_T("PASSWORD"))!=0)
        )
    {
        this->ControlType=SUPPORTED_CTRL_EDIT;
        // if password TCHAR is != 0
        if (SendMessage(this->pSelectWindow->WindowHandle,(UINT) EM_GETPASSWORDCHAR,0,0))
		{
            // it is a password
            this->ControlType=SUPPORTED_CTRL_EDIT_PASSWORD;
		}
    }    

    free(pszClassNameUpper);
    return this->ControlType;
}

//-----------------------------------------------------------------------------
// Name: OpenFile
// Object: open file, check control handle and find file type
// Parameters :
//     in  : 
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::OpenFile(TCHAR* pszFileName)
{
    this->pszFileName=pszFileName;
    TCHAR psz[2*MAX_PATH];

    // NULL pointer
    if (!pszFileName)
        return FALSE;

    // Empty File name
    if (!*pszFileName)
        return FALSE;

    // No control selected
    if (!this->pSelectWindow)
    {
        this->ShowError(_T("No control selected"));
        return FALSE;
    }

    /////////////////////////
    // check extension
    /////////////////////////
    if (CStdFileOperations::DoesExtensionMatch(pszFileName,_T("xml")))
        this->SavingType=CRemoteCtrlContentSaver::REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML;
    else if (CStdFileOperations::DoesExtensionMatch(pszFileName,_T("html")))
        this->SavingType=CRemoteCtrlContentSaver::REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML;
    else
        this->SavingType=CRemoteCtrlContentSaver::REMOTECTRLCONTENTSAVER_SAVING_TYPE_TXT;

    /////////////////////////
    // open file 
    /////////////////////////
    
    if (!CTextFile::CreateTextFile(pszFileName,&this->hFile))
    {
        _tcscpy(psz,_T("Can't create file "));
        _tcscat(psz,pszFileName);
        this->ShowError(psz);
        return FALSE;
    }

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: CloseFile
// Object: close open file and show operation success message if bSuccess.
// Parameters :
//     in  : 
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::CloseFile(BOOL bSuccess)
{
    /////////////////////////
    // Close file
    /////////////////////////
    CloseHandle(this->hFile);

    /////////////////////////
    // Show success message if necessary
    /////////////////////////
    if (bSuccess)
    {
        if (this->bShowMessages)
        {
            if (MessageBox(this->HwndDialog,_T("Save Successfully Completed\r\n\r\nDo you want to view results now ?"),_T("Information"),MB_YESNO|MB_ICONINFORMATION|MB_TOPMOST)==IDYES)
            {
                if (((int)ShellExecute(NULL,_T("open"),this->pszFileName,NULL,NULL,SW_SHOWNORMAL))<=32)
                    this->ShowError(_T("Error opening file"));
            }
        }
    }
    else
        DeleteFile(this->pszFileName);
    /////////////////////////
    // return result
    /////////////////////////
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: ShouldColumnBeSaved
// Object: Check if columns number belongs to the pNumColumns array
// Parameters :
//     in  : DWORD dwColumnNumberToCheck : column number to check
//           DWORD* pNumColumns : array of column numbers that must be saved
//           DWORD dwNbSelectedColomns : pNumColumns array size
//     out : 
//     return : TRUE if columns must be saved
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::ShouldColumnBeSaved(SIZE_T ColumnNumberToCheck,SIZE_T* pNumColumns,SIZE_T NbSelectedColomns)
{
    for (DWORD dwCnt=0;dwCnt<NbSelectedColomns;dwCnt++)
    {
        if (ColumnNumberToCheck==pNumColumns[dwCnt])
            return TRUE;
    }
    return FALSE;
}

//-----------------------------------------------------------------------------
// Name: GetTextContent
// Object: Send the WM_GETTEXT to selected control
// Parameters :
//     in  : 
//     out : TCHAR** pszOut : *pszOut contains text. It is allocated inside the
//                          function, so you need to call free(*pszOut); after this func
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::GetTextContent(TCHAR** pszOut)
{
    TCHAR* psz;
    DWORD dwSize=0;

    if (!this->pSelectWindow)
        return FALSE;

    // just send WM_GETTEXT and WM_GETTEXTLENGTH messages and hope it works
    dwSize=(DWORD)SendMessage(this->pSelectWindow->WindowHandle,WM_GETTEXTLENGTH,0,0);
    if (dwSize==0)
    {
        if (this->bShowMessages)
            MessageBox(this->HwndDialog,_T("Warning can't retreive text or text empty"),_T("Warning"),MB_OK|MB_ICONWARNING|MB_TOPMOST);
        return FALSE;
    }

    psz=(TCHAR*)malloc((dwSize+1)*sizeof(TCHAR));
    *psz=0;
    if(SendMessage(this->pSelectWindow->WindowHandle,WM_GETTEXT,dwSize+1,(LPARAM)psz)==0)
    {
        if (this->bShowMessages)
            MessageBox(this->HwndDialog,_T("Warning can't retreive text"),_T("Warning"),MB_OK|MB_ICONWARNING|MB_TOPMOST);
        return FALSE;
    }

    *pszOut=psz;

    return TRUE;
}


//-----------------------------------------------------------------------------
// Name: GetTextContent
// Object: Send the WM_GETTEXT to selected control, inside the control process memory
// Parameters :
//     in  : 
//     out : TCHAR** pszOut : *pszOut contains text. It is allocated inside the
//                          function, so you need to call free(*pszOut); after this func
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::GetPasswordContent(TCHAR** pszOut)
{
    if (!this->pSelectWindow)
        return FALSE;

    ////////////////////////////////////////////////////
    // send WM_GETTEXT from remote process
    ////////////////////////////////////////////////////
    BOOL bRet=TRUE;

    CCodeInject* pCodeInject;
    SIZE_T ThreadProcCodeSize;

    WCHAR* psz;
    WCHAR* pszLocalText=NULL;
    DWORD dwSize=0;
    SIZE_T ExchangedSize=0;

    // just send WM_GETTEXT and WM_GETTEXTLENGTH messages and hope it works
    dwSize=(DWORD)::SendMessageW(this->pSelectWindow->WindowHandle,WM_GETTEXTLENGTH,0,0);
    if (dwSize==0)
    {
        if (this->bShowMessages)
            ::MessageBox(this->HwndDialog,_T("Warning can't retreive text or text empty"),_T("Warning"),MB_OK|MB_ICONWARNING|MB_TOPMOST);
        return FALSE;
    }
    // open process
    hProcess = ::OpenProcess(
                                PROCESS_CREATE_THREAD     |   // Required for thread creation
                                PROCESS_QUERY_INFORMATION |   // Required by Alpha
                                PROCESS_VM_OPERATION      |   // For VirtualAllocEx/VirtualFreeEx
                                PROCESS_VM_WRITE          |   // For WriteProcessMemory
                                PROCESS_VM_READ,              // For ReadProcessMemory
                                FALSE, 
                                this->pSelectWindow->WindowProcessID
                            );

    if (!hProcess)
    {
        this->ShowError(_T("Can't open process of selected control"));
        return FALSE;
    }


    // allocate a remote buffer
    psz=(WCHAR*)VirtualAllocEx(hProcess, NULL, (dwSize+1)*sizeof(WCHAR), MEM_COMMIT, PAGE_READWRITE);
    if (!psz)
    {
        this->ShowError(_T("Error allocating memory in remote process of selected control"));
        CloseHandle(hProcess);
        return FALSE;
    }

    pCodeInject=new CCodeInject(this->pSelectWindow->WindowProcessID);
    // compute code size
    ThreadProcCodeSize = ((LPBYTE) SendMessageAfterThreadProc - (LPBYTE) SendMessageThreadProc);

	BOOL bFreeNeedToFreeLibrary;//useless as user32must be loaded to display user interface
	HMODULE hRemoteUser32;
	pfSendMessage pSendMessage;
	pCodeInject->RemoteGetModuleHandle(_T("user32.dll"),&hRemoteUser32,&bFreeNeedToFreeLibrary);
	pCodeInject->RemoteGetProcAddress(hRemoteUser32,_T("SendMessageW"),(FARPROC*)&pSendMessage);
	if (!pSendMessage)
	{
		bRet=FALSE;
		goto CleanUp;
	}

    // fill injected data struct infos
    DATASENDMESSAGE data;
    data.Hwnd=this->pSelectWindow->WindowHandle;
    data.pSendMessage=pSendMessage;
    data.Msg=WM_GETTEXT;
    data.Wparam=dwSize+1;
    data.Lparam=(LPARAM)psz;
    
	if (!pCodeInject->SetParameter(&data,sizeof(data)))
	{
		this->ShowError(_T("Error injecting parameter in remote process"));
		bRet=FALSE;
		goto CleanUp;
	}
	if (!pCodeInject->SetCode((FARPROC)SendMessageThreadProc,ThreadProcCodeSize))
	{
		this->ShowError(_T("Error injecting code in remote process"));
		bRet=FALSE;
		goto CleanUp;
	}
	if (!pCodeInject->Execute())
	{
		this->ShowError(_T("Error creating remote thread"));
		bRet=FALSE;
		goto CleanUp;
	}

CleanUp:

    delete pCodeInject;

    if (bRet)
    {
        pszLocalText=new WCHAR[dwSize+1];
        bRet=ReadProcessMemory(hProcess,psz,pszLocalText,(dwSize+1)*sizeof(WCHAR),&ExchangedSize);
    }

    // free allocated memory
    VirtualFreeEx(hProcess,psz,0,MEM_RELEASE);

    CloseHandle(hProcess);

    if (bRet)
#if (defined(UNICODE)||defined(_UNICODE))
        *pszOut=_tcsdup(pszLocalText);
#else
        CAnsiUnicodeConvert::UnicodeToAnsi(pszLocalText,pszOut);
#endif
    if (pszLocalText)
        delete[] pszLocalText;

    return bRet;
}

//-----------------------------------------------------------------------------
// Name: ShowPasswordContent
// Object: Show (and Hide) content of selected password control
// Parameters :
//     in  : 
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::ShowPasswordContent()
{
    if (!this->pSelectWindow)
        return FALSE;

    ////////////////////////////////////////////////////
    // make the following in remote process
    // SendMessage(this->pSelectWindow->WindowHandle,(UINT) EM_SETPASSWORDCHAR,0,0);
    ////////////////////////////////////////////////////
    BOOL bRet=TRUE;

    CCodeInject* pCodeInject;
    SIZE_T ThreadProcCodeSize;
    
    WCHAR wcLastPasswordChar=0;

    // do the EM_GETPASSWORDCHAR with SendMessageW func
    wcLastPasswordChar=(WCHAR)::SendMessageW(this->pSelectWindow->WindowHandle,(UINT) EM_GETPASSWORDCHAR,0,0);

    // open process
    hProcess = OpenProcess(
                                PROCESS_CREATE_THREAD     |   // Required for thread creation
                                PROCESS_QUERY_INFORMATION |   // Required by Alpha
                                PROCESS_VM_OPERATION      |   // For VirtualAllocEx/VirtualFreeEx
                                PROCESS_VM_WRITE          |   // For WriteProcessMemory
                                PROCESS_VM_READ,              // For ReadProcessMemory
                                FALSE, 
                                this->pSelectWindow->WindowProcessID
                            );

    if (!hProcess)
    {
        this->ShowError(_T("Can't open process of selected control"));
        return FALSE;
    }


    pCodeInject=new CCodeInject(this->pSelectWindow->WindowProcessID);
    // compute code size
    ThreadProcCodeSize = ((LPBYTE) SendMessageAfterThreadProc - (LPBYTE) SendMessageThreadProc);

	BOOL bFreeNeedToFreeLibrary;//useless as user32must be loaded to display user interface
	HMODULE hRemoteUser32;
	pfSendMessage pSendMessage;
	pCodeInject->RemoteGetModuleHandle(_T("user32.dll"),&hRemoteUser32,&bFreeNeedToFreeLibrary);
	pCodeInject->RemoteGetProcAddress(hRemoteUser32,_T("SendMessageW"),(FARPROC*)&pSendMessage);
	if (!pSendMessage)
	{
		bRet=FALSE;
		goto CleanUp;
	}

    // fill inject data struct infos
    DATASENDMESSAGE data;
    data.Hwnd=this->pSelectWindow->WindowHandle;
    data.pSendMessage=pSendMessage;
    data.Msg=EM_SETPASSWORDCHAR;
    data.Wparam=0;
    data.Lparam=0;
    

    if (!pCodeInject->SetParameter(&data,sizeof(data)))
    {
        this->ShowError(_T("Error injecting parameter in remote process"));
        bRet=FALSE;
        goto CleanUp;
    }
    if (!pCodeInject->SetCode((FARPROC)SendMessageThreadProc,ThreadProcCodeSize))
    {
        this->ShowError(_T("Error injecting code in remote process"));
        bRet=FALSE;
        goto CleanUp;
    }
    if (!pCodeInject->Execute())
    {
        this->ShowError(_T("Error creating remote thread"));
        bRet=FALSE;
        goto CleanUp;
    }
    ////////////////////////////////////////////////////
    // only EM_SETPASSWORDCHAR message needs to be injected so call the redraw window here
    ////////////////////////////////////////////////////
    RedrawWindow( this->pSelectWindow->WindowHandle,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE);

    if (MessageBox(this->HwndDialog,_T("Just do what you need, and when you've finished,\r\njust click the \"OK\" button to hide password again.\r\nClick \"Cancel\" to let it visible."),_T("Information"),MB_TOPMOST|MB_OKCANCEL|MB_ICONQUESTION)
        ==IDOK)
    {
        // until EM_SETPASSWORDCHAR can be sent to a remote process window with current PASSWORDCHAR defined to 0
        pSendMessage(this->pSelectWindow->WindowHandle,(UINT) EM_SETPASSWORDCHAR,wcLastPasswordChar,0);

        // and if one day it is protect do it using remote process
        // // hide password
        // data.Wparam=wcLastPasswordChar;
        // pCodeInject->SetParameter(&data,sizeof(data));
        // pCodeInject->Execute();

        ////////////////////////////////////////////////////
        // only EM_SETPASSWORDCHAR message needs to be injected so call the redraw window here
        ////////////////////////////////////////////////////
        RedrawWindow( this->pSelectWindow->WindowHandle,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE);
    }

CleanUp:

    delete pCodeInject;
    CloseHandle(hProcess);
    return bRet;
}

//-----------------------------------------------------------------------------
// Name: SaveHeaderContent
// Object: Save content of selected header control
// Parameters :
//     in  : TCHAR* pszFileName : output file name
//           DWORD* pNumColumns : array of columns index to be saved
//           DWORD dwNbSelectedColomns : array of columns index size
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::SaveHeaderContent(TCHAR* pszFileName,SIZE_T* pNumColumns,SIZE_T NbSelectedColomns)
{
    if (!this->pSelectWindow)
        return FALSE;

    if (!this->OpenFile(pszFileName))
        return FALSE;

    BOOL bRet;

    // open process
    hProcess = OpenProcess(
                                PROCESS_QUERY_INFORMATION |   // Required by Alpha
                                PROCESS_VM_OPERATION      |   // For VirtualAllocEx/VirtualFreeEx
                                PROCESS_VM_WRITE          |   // For WriteProcessMemory
                                PROCESS_VM_READ,              // For ReadProcessMemory
                                FALSE, 
                                this->pSelectWindow->WindowProcessID
                            );

    if (!hProcess)
    {
        this->ShowError(_T("Can't open process of selected control"));
        return FALSE;
    }

    if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
    {
        CTextFile::WriteText(this->hFile,_T("<HTML><BODY>"));
        CTextFile::WriteText(this->hFile,_T("<TABLE BORDER=\"1\">"));
    }

    // save header content
    bRet=this->SaveHeaderContent(hProcess,this->pSelectWindow->WindowHandle,pNumColumns,NbSelectedColomns);

    if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
    {
        CTextFile::WriteText(this->hFile,_T("</TABLE>"));
        CTextFile::WriteText(this->hFile, _T("</BODY></HTML>"));
    }

    // close process
    CloseHandle(hProcess);

    if (!this->CloseFile(bRet))
        return FALSE;

    return bRet;
}


//-----------------------------------------------------------------------------
// Name: SaveHeaderContent
// Object: Save content of selected header control
// Parameters :
//     in  : HANDLE hProcess : handle to process
//           HWND hHeader : handle to header control
//           DWORD* pNumColumns : array of columns index to be saved
//           DWORD dwNbSelectedColomns : array of columns index size
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::SaveHeaderContent(HANDLE hProcess,HWND hHeader,SIZE_T* pNumColumns,SIZE_T NbSelectedColomns)
{
    HDITEM* pHDITEM;
    int iNbColumn;
    SIZE_T ExchangedSize;
    BOOL bEmptyOrErrorColumnItem;
    TCHAR* pszRemoteText;// local to function only to avoid interaction with SaveListViewContent
    DWORD pcLocalTextSize;// local to function only to avoid interaction with SaveHeaderContent
    pcLocalTextSize=REMOTECTRLCONTENTSAVER_DEFAULT_TEXT_BUFFER_SIZE;
    
    if (!hHeader)
        return FALSE;

    TCHAR* pcLocalText =new TCHAR[pcLocalTextSize];

    pHDITEM=(HDITEM*)VirtualAllocEx(hProcess, NULL, sizeof(HDITEM), MEM_COMMIT, PAGE_READWRITE);
    if (!pHDITEM)
    {
        this->ShowError(_T("Error allocating memory in remote process of selected control"));
        delete[] pcLocalText;
        return FALSE;
    }

    pszRemoteText=(TCHAR*)VirtualAllocEx(hProcess, NULL, pcLocalTextSize*sizeof(TCHAR), MEM_COMMIT, PAGE_READWRITE);
    if (!pszRemoteText)
    {
        VirtualFreeEx(hProcess,pHDITEM,0,MEM_RELEASE);
        this->ShowError(_T("Error allocating memory in remote process of selected control"));
        delete[] pcLocalText;
        return FALSE;
    }

    ///////////////////////////////////////////////////////////////////
    // Fill struct info in remote memory
    ///////////////////////////////////////////////////////////////////

    // memset(pHDITEM,0,sizeof(HDITEM))
    HDITEM hd={0};
    ExchangedSize=0;
    WriteProcessMemory(hProcess, pHDITEM,(PBYTE)&hd, sizeof(HDITEM), &ExchangedSize);

    // pHDITEM->cchTextMax=Size;
    WriteProcessMemory(hProcess, &(pHDITEM->cchTextMax),(LPCVOID)&pcLocalTextSize, sizeof(int), &ExchangedSize);

    // pHDITEM->pszText=pszRemoteText;
    WriteProcessMemory(hProcess, &(pHDITEM->pszText),(LPCVOID)&pszRemoteText, sizeof(TCHAR*), &ExchangedSize);

    // pHDITEM->Mask HDI_TEXT
    UINT uiMask=HDI_TEXT;
    WriteProcessMemory(hProcess, &(pHDITEM->mask),(LPCVOID)&uiMask, sizeof(UINT), &ExchangedSize);

    // get number of header
    iNbColumn=Header_GetItemCount(hHeader);

    if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
        CTextFile::WriteText(this->hFile, _T("<HEADER>"));
    else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
        CTextFile::WriteText(this->hFile, _T("<TR>"));

    for (int Index=0;Index<iNbColumn;Index++)
    {
        // if only some columns should be saved been 
        if (NbSelectedColomns>0)
        {
            // if column number is not one of the wanted
            if (!this->ShouldColumnBeSaved(Index,pNumColumns,NbSelectedColomns))
                continue;
        }

        bEmptyOrErrorColumnItem=TRUE;

GetData:
        if(SendMessage(hHeader,(UINT)HDM_GETITEM,(WPARAM)Index,(LPARAM)pHDITEM))
        {
            if (ReadProcessMemory(hProcess,pszRemoteText,pcLocalText,pcLocalTextSize,&ExchangedSize))
            {
                if (_tcslen(pcLocalText)+1>=pcLocalTextSize)
                {
                    // allocated buffer was to small

                    TCHAR* NewRemoteMemory;
                    DWORD NewRemoteMemorySize;
                    NewRemoteMemorySize=pcLocalTextSize*2;

                    // allocate remote memory
                    NewRemoteMemory=(TCHAR*)VirtualAllocEx(hProcess, NULL, NewRemoteMemorySize*sizeof(TCHAR), MEM_COMMIT, PAGE_READWRITE);
                    if (!NewRemoteMemory)
                        // continue we can't get full data
                        goto ContinueWithExistingData;

                    // update remote pHDITEM informations
                    // pHDITEM->pszText=pszRemoteText;
                    if (!WriteProcessMemory(hProcess, &(pHDITEM->pszText),(LPCVOID)&NewRemoteMemory, sizeof(TCHAR*), &ExchangedSize))
                    {
                        VirtualFreeEx(this->hProcess,NewRemoteMemory,0,MEM_RELEASE);
                        // continue we can't get full data
                        goto ContinueWithExistingData;
                    }
                    // free previously allocated remote memory
                    VirtualFreeEx(this->hProcess,pszRemoteText,0,MEM_RELEASE);

                    // point to new remote memory
                    pszRemoteText=NewRemoteMemory;

                    // pHDITEM->cchTextMax=Size;
                    if (!WriteProcessMemory(hProcess, &(pHDITEM->cchTextMax),(LPCVOID)&NewRemoteMemorySize, sizeof(int), &ExchangedSize))
                        // continue we can't get full data
                        goto ContinueWithExistingData;

                    // update local buffer size
                    pcLocalTextSize=NewRemoteMemorySize;

                    // free local memory
                    delete[] pcLocalText;

                    pcLocalText=new TCHAR[pcLocalTextSize];
                    
                    goto GetData;

                }
ContinueWithExistingData:

                bEmptyOrErrorColumnItem=FALSE;
                TCHAR* pszXMLLike=new TCHAR[REPLACED_CHAR_MAX_INCREASE*_tcslen(pcLocalText)+1];
                if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
                {
                    CTextFile::WriteText(this->hFile, _T("<COLUMN>"));
                    CXMLLikeConvert::Convert(pcLocalText,pszXMLLike,TRUE);
                    CTextFile::WriteText(this->hFile, pszXMLLike);
                    CTextFile::WriteText(this->hFile,_T("</COLUMN>") );
                }
                else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
                {
                    CTextFile::WriteText(this->hFile, _T("<TD>"));
                    CXMLLikeConvert::Convert(pcLocalText,pszXMLLike,TRUE);
                    CTextFile::WriteText(this->hFile, pszXMLLike);
                    CTextFile::WriteText(this->hFile, _T("</TD>"));
                }
                else
                {
                    CTextFile::WriteText(this->hFile, pcLocalText);
                    // if not last column
                    if (Index!=(iNbColumn-1))
                        CTextFile::WriteText(this->hFile, TXT_FILE_SPLITTER);
                }
                delete[] pszXMLLike;
            }
        }
        if (bEmptyOrErrorColumnItem)
        {
            if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
                CTextFile::WriteText(this->hFile, _T("<COLUMN/>"));
            else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
                CTextFile::WriteText(this->hFile, _T("<TD></TD>"));
            else
            {
                // if not last column
                if (Index!=(iNbColumn-1))
                    CTextFile::WriteText(this->hFile, TXT_FILE_SPLITTER);
            }
        }
    }

    if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
        CTextFile::WriteText(this->hFile, _T("</HEADER>"));
    else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
        CTextFile::WriteText(this->hFile, _T("</TR>"));
    else
        CTextFile::WriteText(this->hFile, _T("\r\n"));

    VirtualFreeEx(hProcess,pHDITEM,0,MEM_RELEASE);
    VirtualFreeEx(hProcess,pszRemoteText,0,MEM_RELEASE);

    delete[] pcLocalText;

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: SaveListViewContent
// Object: Save content of selected listview control
// Parameters :
//     in  : TCHAR* pszFileName : output file name
//           BOOL bSelectedItemsOnly : TRUE to save only selected items
//           DWORD* pNumColumns : array of columns index to be saved
//           DWORD dwNbSelectedColomns : array of columns index size
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::SaveListViewContent(TCHAR* pszFileName,BOOL bSelectedItemsOnly,SIZE_T* pNumColumns,SIZE_T NbSelectedColomns)
{
    if (!this->pSelectWindow)
        return FALSE;

    if (!this->OpenFile(pszFileName))
        return FALSE;

    int iNbRow;
    int iNbColumn;
    HWND hHeader;

    int ItemIndex;
    int SubItemIndex;

    TCHAR* pszRemoteText;// local to function only to avoid interaction with SaveHeaderContent
    DWORD pcLocalTextSize;// local to function only to avoid interaction with SaveHeaderContent
    pcLocalTextSize=REMOTECTRLCONTENTSAVER_DEFAULT_TEXT_BUFFER_SIZE;
    LRESULT lResult;
    SIZE_T ExchangedSize;
    LVITEM* pLVITEM;
    TCHAR* pcLocalText=new TCHAR[pcLocalTextSize];
    BOOL bEmptyOrErrorColumnItem;

    hProcess = OpenProcess(
                                PROCESS_QUERY_INFORMATION |   // Required by Alpha
                                PROCESS_VM_OPERATION      |   // For VirtualAllocEx/VirtualFreeEx
                                PROCESS_VM_WRITE          |   // For WriteProcessMemory
                                PROCESS_VM_READ,              // For ReadProcessMemory
                                FALSE, 
                                this->pSelectWindow->WindowProcessID
                            );

    if (!hProcess)
    {
        this->ShowError(_T("Can't open process of selected control"));
        delete[] pcLocalText;
        return FALSE;
    }

    ///////////////////////////////////////////////////////////////////
    // allocate pointer required by LVM_GETITEMTEXT in other process memory
    ///////////////////////////////////////////////////////////////////
    pLVITEM=(LVITEM*)VirtualAllocEx(hProcess, NULL, sizeof(LVITEM), MEM_COMMIT, PAGE_READWRITE);
    if (!pLVITEM)
    {
        CloseHandle(hProcess);
        this->ShowError(_T("Error allocating memory in remote process of selected control"));
        delete[] pcLocalText;
        return FALSE;
    }

    pszRemoteText=(TCHAR*)VirtualAllocEx(hProcess, NULL, pcLocalTextSize*sizeof(TCHAR), MEM_COMMIT, PAGE_READWRITE);
    if (!pszRemoteText)
    {
        VirtualFreeEx(hProcess,pLVITEM,0,MEM_RELEASE);
        CloseHandle(hProcess);
        this->ShowError(_T("Error allocating memory in remote process of selected control"));
        delete[] pcLocalText;
        return FALSE;
    }

    ///////////////////////////////////////////////////////////////////
    // Fill struct info in remote memory
    ///////////////////////////////////////////////////////////////////

    // memset(pLVITEM,0,sizeof(LVITEM))
    LVITEM lv={0};
    ExchangedSize=0;
    WriteProcessMemory(hProcess, pLVITEM,(PBYTE)&lv, sizeof(LVITEM), &ExchangedSize);

    // pLVITEM->cchTextMax=Size;
    WriteProcessMemory(hProcess, &(pLVITEM->cchTextMax),(LPCVOID)&pcLocalTextSize, sizeof(int), &ExchangedSize);

    // pLVITEM->pszText=pszRemoteText;
    WriteProcessMemory(hProcess, &(pLVITEM->pszText),(LPCVOID)&pszRemoteText, sizeof(TCHAR*), &ExchangedSize);

    if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
        CTextFile::WriteText(this->hFile, _T("<LISTVIEW>"));

    else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
    {
        CTextFile::WriteText(this->hFile, _T("<HTML><BODY>"));
        CTextFile::WriteText(this->hFile, _T("<TABLE BORDER=\"1\">"));
    }

    ///////////////////////////////////////////////////////////////////
    // get row and column number
    ///////////////////////////////////////////////////////////////////
    iNbRow=ListView_GetItemCount(this->pSelectWindow->WindowHandle);
    iNbColumn=1;
    hHeader = ListView_GetHeader(this->pSelectWindow->WindowHandle);
    if (hHeader)
    {
        iNbColumn=Header_GetItemCount(hHeader);
        this->SaveHeaderContent(hProcess,hHeader,pNumColumns,NbSelectedColomns);
    }


    // for each raw
    for (ItemIndex=0;ItemIndex<iNbRow;ItemIndex++)
    {
        if (bSelectedItemsOnly)
        {
            // if item is not selected go to next item
            if (ListView_GetItemState(this->pSelectWindow->WindowHandle,ItemIndex,LVIS_SELECTED)!=LVIS_SELECTED)
                continue;
        }
        if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
            CTextFile::WriteText(this->hFile, _T("<ROW>"));
        else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
            CTextFile::WriteText(this->hFile, _T("<TR>"));
        // for each column
        for (SubItemIndex=0;SubItemIndex<iNbColumn;SubItemIndex++)
        {
            // if only some columns should be saved been 
            if (NbSelectedColomns>0)
            {
                // if column number is not one of the wanted
                if (!this->ShouldColumnBeSaved(SubItemIndex,pNumColumns,NbSelectedColomns))
                    continue;
            }

            bEmptyOrErrorColumnItem=TRUE;
            ///////////////////////////////////////////////////////////////////
            // Fill struct info (subitem index) in remote memory
            ///////////////////////////////////////////////////////////////////
            // pLVITEM->iSubItem=SubItemIndex;
            WriteProcessMemory(hProcess, &(pLVITEM->iSubItem),(LPCVOID)&SubItemIndex, sizeof(int), &ExchangedSize);

GetData:
            ///////////////////////////////////////////////////////////////////
            // Send LVM_GETITEMTEXT message lResult will contain the number of TCHAR copied
            ///////////////////////////////////////////////////////////////////
            lResult = SendMessage((HWND) this->pSelectWindow->WindowHandle,      // handle to destination control
            (UINT) LVM_GETITEMTEXT,   // message ID
            (WPARAM) ItemIndex,       // = (WPARAM) (int) iItem;
            (LPARAM) pLVITEM          // = (LPARAM) (LPLVITEM) pitem;
            );  

 
            if (lResult>0)
            {
                ///////////////////////////////////////////////////////////////////
                // read remote process memory containing data
                ///////////////////////////////////////////////////////////////////
                if (ReadProcessMemory(hProcess,pszRemoteText,pcLocalText,lResult*sizeof(TCHAR),&ExchangedSize))
                {
                    // ends string
                    pcLocalText[lResult]=0;

                    if (_tcslen(pcLocalText)+1>=pcLocalTextSize)
                    {
                        // allocated buffer was to small

                        TCHAR* NewRemoteMemory;
                        DWORD NewRemoteMemorySize;
                        NewRemoteMemorySize=pcLocalTextSize*2;

                        // allocate remote memory
                        NewRemoteMemory=(TCHAR*)VirtualAllocEx(hProcess, NULL, NewRemoteMemorySize*sizeof(TCHAR), MEM_COMMIT, PAGE_READWRITE);
                        if (!NewRemoteMemory)
                            // continue we can't get full data
                            goto ContinueWithExistingData;

                        // update remote pLVITEM informations
                        // pLVITEM->pszText=pszRemoteText;
                        if (!WriteProcessMemory(hProcess, &(pLVITEM->pszText),(LPCVOID)&NewRemoteMemory, sizeof(TCHAR*), &ExchangedSize))
                        {
                            VirtualFreeEx(this->hProcess,NewRemoteMemory,0,MEM_RELEASE);
                            // continue we can't get full data
                            goto ContinueWithExistingData;
                        }
                        // free previously allocated remote memory
                        VirtualFreeEx(this->hProcess,pszRemoteText,0,MEM_RELEASE);

                        // point to new remote memory
                        pszRemoteText=NewRemoteMemory;

                        // pLVITEM->cchTextMax=Size;
                        if (!WriteProcessMemory(hProcess, &(pLVITEM->cchTextMax),(LPCVOID)&NewRemoteMemorySize, sizeof(int), &ExchangedSize))
                            // continue we can't get full data
                            goto ContinueWithExistingData;

                        // update local buffer size
                        pcLocalTextSize=NewRemoteMemorySize;

                        // free local memory
                        delete[] pcLocalText;

                        pcLocalText=new TCHAR[pcLocalTextSize];
                        
                        goto GetData;

                    }
ContinueWithExistingData:


                    bEmptyOrErrorColumnItem=FALSE;
                    // ends string
                    pcLocalText[lResult]=0;

                    TCHAR* pszXMLLike=new TCHAR[REPLACED_CHAR_MAX_INCREASE*_tcslen(pcLocalText)+1];
                    ///////////////////////////////////////////////////////////////////
                    // write item data
                    ///////////////////////////////////////////////////////////////////
                    if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
                    {
                        CTextFile::WriteText(this->hFile, _T("<COLUMN>"));
                        CXMLLikeConvert::Convert(pcLocalText,pszXMLLike,TRUE);
                        CTextFile::WriteText(this->hFile, pszXMLLike);
                        CTextFile::WriteText(this->hFile, _T("</COLUMN>"));
                    }
                    else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
                    {
                        CTextFile::WriteText(this->hFile, _T("<TD>"));
                        CXMLLikeConvert::Convert(pcLocalText,pszXMLLike,TRUE);
                        CTextFile::WriteText(this->hFile, pszXMLLike);
                        CTextFile::WriteText(this->hFile, _T("</TD>"));
                    }
                    else
                    {
                        CTextFile::WriteText(this->hFile, pcLocalText);
                        // if not last column
                        if (SubItemIndex!=(iNbColumn-1))
                            CTextFile::WriteText(this->hFile, TXT_FILE_SPLITTER);
                    }
                    delete[] pszXMLLike;
                }
            }
            if (bEmptyOrErrorColumnItem)
            {
                if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
                    CTextFile::WriteText(this->hFile, _T("<COLUMN/>"));
                else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
                    CTextFile::WriteText(this->hFile, _T("<TD></TD>"));
                else
                {
                    // if not last column
                    if (SubItemIndex!=(iNbColumn-1))
                        CTextFile::WriteText(this->hFile, TXT_FILE_SPLITTER);
                }
            }
        }
        if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
            CTextFile::WriteText(this->hFile, _T("</ROW>"));
        else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
            CTextFile::WriteText(this->hFile, _T("</TR>\r\n"));
        else
            CTextFile::WriteText(this->hFile, _T("\r\n"));
    }
    if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
        CTextFile::WriteText(this->hFile, _T("</LISTVIEW>"));
    else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
    {
        CTextFile::WriteText(this->hFile, _T("</TABLE>"));
        CTextFile::WriteText(this->hFile, _T("</BODY></HTML>"));
    }


    ////////////////////////////////////////////
    // free allocated memory
    ////////////////////////////////////////////
    VirtualFreeEx(hProcess,pLVITEM,0,MEM_RELEASE);
    VirtualFreeEx(hProcess,pszRemoteText,0,MEM_RELEASE);
    CloseHandle(hProcess);

    delete[] pcLocalText;

    if (!this->CloseFile(TRUE))
        return FALSE;

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: SaveComboBoxEx32Content
// Object: Save content of selected ComboBoxEx32 control
// Parameters :
//     in  : TCHAR* pszFileName : output file name
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::SaveComboBoxEx32Content(TCHAR* pszFileName)
{
    if (!this->pSelectWindow)
        return FALSE;

    // save handle
    HWND hCombobox=this->pSelectWindow->WindowHandle;
    // Other messages must be send to the comboboxex which is the parent of the combobox
    // get parent window info
    this->pSelectWindow->GetHandleInfo(this->pSelectWindow->ParentWindowHandle);
    BOOL bRet=this->SaveComboBoxContent(pszFileName);

    // restore handle
    this->pSelectWindow->WindowHandle=hCombobox;
    this->pSelectWindow->GetHandleInfo(this->pSelectWindow->WindowHandle);
    return bRet;
}

//-----------------------------------------------------------------------------
// Name: SaveComboBoxContent
// Object: Save content of selected ComboBox control
// Parameters :
//     in  : TCHAR* pszFileName : output file name
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::SaveComboBoxContent(TCHAR* pszFileName)
{
    if (!this->pSelectWindow)
        return FALSE;

    if (!this->OpenFile(pszFileName))
        return FALSE;

    TCHAR* psz=NULL;
    DWORD dwSize=0;
    DWORD dwMaxSize=0;
    int ItemIndex;
    int NbItems;
    TCHAR* pszXMLLike;

    // get number of items
    NbItems=(int)SendMessage(this->pSelectWindow->WindowHandle,CB_GETCOUNT,0,0);
    if (NbItems<=1)
    {
        // send messages to the combobox to assume items are filled because in some app (only those using ComboBoxEx ?),
        // items are inserted only at the first left mouse button down
        SendMessage(this->pSelectWindow->WindowHandle,WM_LBUTTONDOWN,MK_LBUTTON,0);
        SendMessage(this->pSelectWindow->WindowHandle,WM_LBUTTONUP,0,0);

        // send messages to the combobox to assume items are filled because in some app (only those using ComboBoxEx ?),
        // items are inserted only at the first Drop down list showing
        SendMessage(this->pSelectWindow->WindowHandle,CB_SHOWDROPDOWN,TRUE,0);
        SendMessage(this->pSelectWindow->WindowHandle,CB_SHOWDROPDOWN,FALSE,0);

        // get number of items
        NbItems=(int)SendMessage(this->pSelectWindow->WindowHandle,CB_GETCOUNT,0,0);
    }
    if (NbItems<=0)
    {
        this->ShowError(_T("Empty ComboBox"));
        return FALSE;
    }

    if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
        CTextFile::WriteText(this->hFile, _T("<COMBOBOX>"));
    else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
    {
        CTextFile::WriteText(this->hFile, _T("<HTML><BODY>"));
        CTextFile::WriteText(this->hFile, _T("<TABLE BORDER=\"1\">"));
    }

    for (ItemIndex=0;ItemIndex<NbItems;ItemIndex++)
    {
        dwSize=(DWORD)SendMessage(this->pSelectWindow->WindowHandle,CB_GETLBTEXTLEN,ItemIndex,0);
        if (dwSize!=0)
        {
            // check buffer size and increase if necessary
            if (dwSize>dwMaxSize)
            {
                if (psz)
                    delete psz;
                psz=new TCHAR[dwSize+1];
                dwMaxSize=dwSize;
            }
            *psz=0;

            // get item text
            if(SendMessage(this->pSelectWindow->WindowHandle,CB_GETLBTEXT,ItemIndex,(LPARAM)psz)!=0)
            {
                pszXMLLike=new TCHAR[_tcslen(psz)*REPLACED_CHAR_MAX_INCREASE+1];

                ///////////////////////////////////////////////////////////////////
                // write item text
                ///////////////////////////////////////////////////////////////////
                if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
                {
                    CTextFile::WriteText(this->hFile, _T("<ITEM>"));
                    CXMLLikeConvert::Convert(psz,pszXMLLike,TRUE);
                    CTextFile::WriteText(this->hFile, pszXMLLike);
                    CTextFile::WriteText(this->hFile, _T("</ITEM>"));
                }
                else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
                {
                    CTextFile::WriteText(this->hFile, _T("<TR>"));
                    CTextFile::WriteText(this->hFile, _T("<TD>"));
                    CXMLLikeConvert::Convert(psz,pszXMLLike,TRUE);
                    CTextFile::WriteText(this->hFile, pszXMLLike);
                    CTextFile::WriteText(this->hFile, _T("</TD>"));
                    CTextFile::WriteText(this->hFile, _T("</TR>\r\n"));
                }
                else
                {
                    CTextFile::WriteText(this->hFile, psz);
                    CTextFile::WriteText(this->hFile, _T("\r\n"));
                }
                delete pszXMLLike;
            }
        }
    }
    if (psz)
        delete psz;

    if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
        CTextFile::WriteText(this->hFile, _T("</COMBOBOX>"));
    else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
    {
        CTextFile::WriteText(this->hFile, _T("</TABLE>"));
        CTextFile::WriteText(this->hFile, _T("</BODY></HTML>"));
    }

    if (!this->CloseFile(TRUE))
        return FALSE;

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: SaveListBoxContent
// Object: Save content of selected ListBox control
// Parameters :
//     in  : TCHAR* pszFileName : output file name
//           BOOL bSelectedItemsOnly : TRUE to save only selected items
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::SaveListBoxContent(TCHAR* pszFileName,BOOL bSelectedItemsOnly)
{
    if (!this->pSelectWindow)
        return FALSE;

    if (!this->OpenFile(pszFileName))
        return FALSE;

    TCHAR* psz=NULL;
    DWORD dwSize=0;
    DWORD dwMaxSize=0;
    int ItemIndex;
    int NbItems=(int)SendMessage(this->pSelectWindow->WindowHandle,LB_GETCOUNT,0,0);
    TCHAR* pszXMLLike;

    if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
        CTextFile::WriteText(this->hFile, _T("<LISTBOX>"));
    else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
    {
        CTextFile::WriteText(this->hFile, _T("<HTML><BODY>"));
        CTextFile::WriteText(this->hFile, _T("<TABLE BORDER=\"1\">"));
    }

    for (ItemIndex=0;ItemIndex<NbItems;ItemIndex++)
    {
        if (bSelectedItemsOnly)
        {
            // if item is not selected go to next item
            if (SendMessage(this->pSelectWindow->WindowHandle,LB_GETSEL,ItemIndex,0)<=0)
                continue;
        }

        dwSize=(DWORD)SendMessage(this->pSelectWindow->WindowHandle,LB_GETTEXTLEN,ItemIndex,0);
        if (dwSize!=0)
        {
            // check buffer size and increase if necessary
            if (dwSize>dwMaxSize)
            {
                if (psz)
                    delete psz;
                psz=new TCHAR[dwSize+1];
                dwMaxSize=dwSize;
            }
            *psz=0;
            // get item text
            if(SendMessage(this->pSelectWindow->WindowHandle,LB_GETTEXT,ItemIndex,(LPARAM)psz)!=0)
            {
                pszXMLLike=new TCHAR[_tcslen(psz)*REPLACED_CHAR_MAX_INCREASE+1];
                ///////////////////////////////////////////////////////////////////
                // write item text
                ///////////////////////////////////////////////////////////////////
                if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
                {
                    CTextFile::WriteText(this->hFile, _T("<ITEM>"));
                    CXMLLikeConvert::Convert(psz,pszXMLLike,TRUE);
                    CTextFile::WriteText(this->hFile, pszXMLLike);
                    CTextFile::WriteText(this->hFile, _T("</ITEM>"));
                }
                else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
                {
                    CTextFile::WriteText(this->hFile, _T("<TR>"));
                    CTextFile::WriteText(this->hFile, _T("<TD>"));
                    CXMLLikeConvert::Convert(psz,pszXMLLike,TRUE);
                    CTextFile::WriteText(this->hFile, pszXMLLike);
                    CTextFile::WriteText(this->hFile, _T("</TD>"));
                    CTextFile::WriteText(this->hFile, _T("</TR>\r\n"));
                }
                else
                {
                    CTextFile::WriteText(this->hFile, psz);
                    CTextFile::WriteText(this->hFile, _T("\r\n"));
                }
                delete pszXMLLike;
            }
        }
    }
    // free memory
    if (psz)
        delete psz;
    
    if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
        CTextFile::WriteText(this->hFile, _T("</LISTBOX>"));
    else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
    {
        CTextFile::WriteText(this->hFile, _T("</TABLE>"));
        CTextFile::WriteText(this->hFile, _T("</BODY></HTML>"));
    }

    if (!this->CloseFile(TRUE))
        return FALSE;

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: GetTreeViewSelectedContent
// Object: get content of selected TreeView control
// Parameters :
//     in  : 
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::GetTreeViewSelectedContent(TCHAR** pszOut)
{
    if (!this->pSelectWindow)
        return FALSE;

    HTREEITEM hTreeItem;
    hTreeItem=TreeView_GetSelection(this->pSelectWindow->WindowHandle);

    if (!hTreeItem)
        return FALSE;

    if (!this->InitializeGetTreeViewItemText())
        return FALSE;

    if (!this->GetTreeViewItemText(hTreeItem,pszOut))
    {
        this->FreeGetTreeViewItemText();
        return FALSE;
    }

    if (!this->FreeGetTreeViewItemText())
        return FALSE;

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: InitializeGetTreeViewItemText
// Object: allocate remote process memory for tree view
// Parameters :
//     in  : 
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::InitializeGetTreeViewItemText()
{
    // open process
    this->hProcess = OpenProcess(
                                PROCESS_CREATE_THREAD     |   // Required for thread creation
                                PROCESS_QUERY_INFORMATION |   // Required by Alpha
                                PROCESS_VM_OPERATION      |   // For VirtualAllocEx/VirtualFreeEx
                                PROCESS_VM_WRITE          |   // For WriteProcessMemory
                                PROCESS_VM_READ,              // For ReadProcessMemory
                                FALSE, 
                                this->pSelectWindow->WindowProcessID
                            );
    if (!hProcess)
    {
        this->ShowError(_T("Can't open process of selected control"));
        return FALSE;
    }


    // allocate memory for a TV_ITEM struct
    this->pRemoteTreeViewItem=(TV_ITEM*)VirtualAllocEx(hProcess, NULL, sizeof(TV_ITEM), MEM_COMMIT, PAGE_READWRITE);
    if (!this->pRemoteTreeViewItem)
    {
        this->ShowError(_T("Error allocating memory in remote process of selected control"));
        CloseHandle(hProcess);
        return FALSE;
    }

    // allocate memory for text retrieval
    this->pRemoteItemText=(TCHAR*)VirtualAllocEx(hProcess, NULL, this->TextBufferSize*sizeof(TCHAR), MEM_COMMIT, PAGE_READWRITE);
    if (!this->pRemoteItemText)
    {
        VirtualFreeEx(this->hProcess,this->pRemoteTreeViewItem,0,MEM_RELEASE);
        this->pRemoteTreeViewItem=NULL;
        this->ShowError(_T("Error allocating memory in remote process of selected control"));
        CloseHandle(hProcess);
        return FALSE;
    }

    // fill remote TV_ITEM information
    TV_ITEM LocalItem={0};
    LocalItem.cchTextMax=this->TextBufferSize;
    LocalItem.pszText=this->pRemoteItemText;
    LocalItem.mask=TVIF_TEXT;

    SIZE_T ExchangedSize;
    if (!WriteProcessMemory(this->hProcess,this->pRemoteTreeViewItem,&LocalItem,sizeof(TV_ITEM),&ExchangedSize))
    {
        this->FreeGetTreeViewItemText();
        return FALSE;
    }

    return TRUE;
}
//-----------------------------------------------------------------------------
// Name: FreeGetTreeViewItemText
// Object: free remote process memory previously allocated for tree view
// Parameters :
//     in  : 
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::FreeGetTreeViewItemText()
{
    // free allocated memory
    if (this->pRemoteTreeViewItem)
    {
        VirtualFreeEx(this->hProcess,this->pRemoteTreeViewItem,0,MEM_RELEASE);
        this->pRemoteTreeViewItem=NULL;
    }
    if (this->pRemoteItemText)
    {
        VirtualFreeEx(this->hProcess,this->pRemoteItemText,0,MEM_RELEASE);
        this->pRemoteItemText=NULL;
    }

    if (this->hProcess)
        CloseHandle(this->hProcess);
    return TRUE;
}
//-----------------------------------------------------------------------------
// Name: GetTreeViewItemText
// Object: get content of specified TreeView item
// Parameters :
//     in  : HTREEITEM hTreeItem
//     out : TCHAR** pszOut must be free by delete
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::GetTreeViewItemText(HTREEITEM hTreeItem,TCHAR** pszOut)
{
    BOOL bRet;
    
    SIZE_T ExchangedSize;

    *pszOut=new TCHAR[this->TextBufferSize];

    // fill TV_ITEM->hItem in remote process
    if (!WriteProcessMemory(this->hProcess,&this->pRemoteTreeViewItem->hItem,&hTreeItem,sizeof(HTREEITEM),&ExchangedSize))
        return FALSE;
GetData:
    // remotely get item
    if (!TreeView_GetItem(this->pSelectWindow->WindowHandle,this->pRemoteTreeViewItem))
        return FALSE;

    // get text in remote process
    bRet= ReadProcessMemory(this->hProcess,this->pRemoteItemText,*pszOut,this->TextBufferSize*sizeof(TCHAR),&ExchangedSize);

    // check if remotely allocated memory is enough
    if (_tcslen(*pszOut)+1>=this->TextBufferSize)
    {
        // memory was not enough

        TCHAR* NewRemoteMemory;
        DWORD NewRemoteMemorySize;
        NewRemoteMemorySize=this->TextBufferSize*2;

        // allocate remote memory
        NewRemoteMemory=(TCHAR*)VirtualAllocEx(hProcess, NULL, NewRemoteMemorySize*sizeof(TCHAR), MEM_COMMIT, PAGE_READWRITE);
        if (!NewRemoteMemory)
            // return retrieved data (not all but the most we can)
            return bRet;

        // update remote TV_ITEM informations
        if (!WriteProcessMemory(this->hProcess,&this->pRemoteTreeViewItem->pszText,&NewRemoteMemory,sizeof(TCHAR*),&ExchangedSize))
        {
            VirtualFreeEx(this->hProcess,NewRemoteMemory,0,MEM_RELEASE);
            return bRet;
        }
        // free previously allocated remote memory
        VirtualFreeEx(this->hProcess,this->pRemoteItemText,0,MEM_RELEASE);

        // point to new remote memory
        this->pRemoteItemText=NewRemoteMemory;

        if (!WriteProcessMemory(this->hProcess,&this->pRemoteTreeViewItem->cchTextMax,&NewRemoteMemorySize,sizeof(int),&ExchangedSize))
            return bRet;

        // update local buffer size
        this->TextBufferSize=NewRemoteMemorySize;

        // free local memory
        delete[] *pszOut;

        *pszOut=new TCHAR[this->TextBufferSize];
        
        goto GetData;
        
    }

    return bRet;
}

//-----------------------------------------------------------------------------
// Name: GetTreeViewItemDepth
// Object: get depth of specified TreeView item
// Parameters :
//     in  : HTREEITEM hTreeItem
//     out : 
//     return : item depth
//-----------------------------------------------------------------------------
DWORD CRemoteCtrlContentSaver::GetTreeViewItemDepth(HTREEITEM hTreeItem)
{
    HTREEITEM hParentTreeItem;
    DWORD ItemDepth=0;
    hParentTreeItem=TreeView_GetParent(this->pSelectWindow->WindowHandle,hTreeItem);
    while(hParentTreeItem)
    {
        ItemDepth++;
        hParentTreeItem=TreeView_GetParent(this->pSelectWindow->WindowHandle,hParentTreeItem);
    }
    return ItemDepth;
}

//-----------------------------------------------------------------------------
// Name: WriteTreeViewItemHeader
// Object: write header for specified item
// Parameters :
//     in  : DWORD ItemDepth : depth of item
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::WriteTreeViewItemHeader(DWORD ItemDepth)
{
    // item header
    if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
        CTextFile::WriteText(this->hFile, _T("<ITEM>"));
    else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
    {
        CTextFile::WriteText(this->hFile, _T("<TABLE>"));
        CTextFile::WriteText(this->hFile, _T("<TR>"));
        CTextFile::WriteText(this->hFile, _T("<TD>"));
    }
    else
    {
        for (DWORD cnt=0;cnt<ItemDepth;cnt++)
            CTextFile::WriteText(this->hFile, TXT_FILE_SPLITTER);
    }

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: WriteTreeViewItemFooter
// Object: write footer for specified item
// Parameters :
//     in  : 
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::WriteTreeViewItemFooter()
{
    // item footer
    if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
        CTextFile::WriteText(this->hFile, _T("</ITEM>"));
    else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
    {
        CTextFile::WriteText(this->hFile, _T("</TD>"));
        CTextFile::WriteText(this->hFile, _T("</TR>"));
        CTextFile::WriteText(this->hFile, _T("</TABLE>"));
    }
    // splitting chars for next item
    CTextFile::WriteText(this->hFile, _T("\r\n"));

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: WriteTreeViewItemContent
// Object: write tree view item content
// Parameters :
//     in  : HTREEITEM hTreeItem : handle to tree view item
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::WriteTreeViewItemContent(HTREEITEM hTreeItem)
{
    TCHAR* psz;
    TCHAR* pszXMLLike;
    psz=NULL;

    if (!this->GetTreeViewItemText(hTreeItem,&psz))
        return FALSE;

    pszXMLLike=new TCHAR[_tcslen(psz)*REPLACED_CHAR_MAX_INCREASE+1];

    ///////////////////////////////////////////////////////////////////
    // write item text
    ///////////////////////////////////////////////////////////////////
    if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
    {
        CTextFile::WriteText(this->hFile, _T("<NAME>"));
        CXMLLikeConvert::Convert(psz,pszXMLLike,TRUE);
        CTextFile::WriteText(this->hFile, pszXMLLike);
        CTextFile::WriteText(this->hFile, _T("</NAME>"));
    }
    else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
    {
        CXMLLikeConvert::Convert(psz,pszXMLLike,TRUE);
        CTextFile::WriteText(this->hFile, pszXMLLike);
    }
    else
    {
        CTextFile::WriteText(this->hFile, psz);
    }

    delete pszXMLLike;

    // free memory
    if (psz)
        delete psz;

    return TRUE;
}


//-----------------------------------------------------------------------------
// Name: SaveTreeViewContent
// Object: Save content of selected TreeView control
// Parameters :
//     in  : TCHAR* pszFileName : output file name
//           BOOL bExpandedOnly : TRUE to save only expanded items
//           BOOL bSelectedOnly : TRUE to save only the selected item
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::SaveTreeViewContent(TCHAR* pszFileName,BOOL bExpandedOnly,BOOL bSelectedOnly)
{
    HTREEITEM hTreeItem;
    BOOL bRet=TRUE;

    if (!this->pSelectWindow)
        return FALSE;

    // allocate memory in remote process
    if (!this->InitializeGetTreeViewItemText())
        return FALSE;

    // open output file
    if (!this->OpenFile(pszFileName))
    {
        this->FreeGetTreeViewItemText();
        return FALSE;
    }

    // write tree view content header
    if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
        CTextFile::WriteText(this->hFile, _T("<TREEVIEW>"));
    else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
    {
        CTextFile::WriteText(this->hFile, _T("<HTML><BODY>"));
        CTextFile::WriteText(this->hFile, _T("<TABLE BORDER=\"1\"><TR><TD>"));
    }

    if (bSelectedOnly)
    {
        // get selected item
        hTreeItem=TreeView_GetSelection(this->pSelectWindow->WindowHandle);

        // write item header
        this->WriteTreeViewItemHeader(0);

        // if item retrieval was successful
        if (hTreeItem)
        {
            // write item content
            this->WriteTreeViewItemContent(hTreeItem);
        }
        else
            bRet=FALSE;

        // write item footer
        this->WriteTreeViewItemFooter();
    }
    else
    {
        // get first item of tree view
        hTreeItem=TreeView_GetRoot(this->pSelectWindow->WindowHandle);
        if(!hTreeItem)
            bRet=FALSE;

        // for each parent item
        while (hTreeItem)
        {
            // save all item data and it's sub items
            this->SaveTreeViewItemContent(hTreeItem,0,bExpandedOnly);

            // get next item having the same parent
            hTreeItem=TreeView_GetNextSibling(this->pSelectWindow->WindowHandle,hTreeItem);
        }
    }

    // write tree view content footer
    if(this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML)
        CTextFile::WriteText(this->hFile, _T("</TREEVIEW>"));
    else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
        CTextFile::WriteText(this->hFile, _T("</TD></TR></TABLE></BODY></HTML>"));

    // free allocated memory
    this->FreeGetTreeViewItemText();

    // close file
    if (!this->CloseFile(TRUE))
        return FALSE;

    return bRet;
}

//-----------------------------------------------------------------------------
// Name: SaveTreeViewItemContent
// Object: Save content of specified hTreeItem and it's children (according to bExpandedOnly)
// Parameters :
//     in  : HTREEITEM hTreeItem : handle to current tree view item
//           DWORD ItemDepth : current item depth
//           BOOL bExpandedOnly : TRUE to get only expanded subitem content
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CRemoteCtrlContentSaver::SaveTreeViewItemContent(HTREEITEM hTreeItem,DWORD ItemDepth,BOOL bExpandedOnly)
{
    if (!hTreeItem)
        return FALSE;

    BOOL bGetSubItems=TRUE; // flag to known if we have to parse children

    // if we get only expanded content
    if (bExpandedOnly)
    {
        // check expanded state
        DWORD ItemState=TreeView_GetItemState(this->pSelectWindow->WindowHandle,hTreeItem,TVIF_STATE);
        bGetSubItems=(ItemState&TVIS_EXPANDED);
    }

    // item header
    this->WriteTreeViewItemHeader(ItemDepth);

    // item content
    this->WriteTreeViewItemContent(hTreeItem);

    // item children
    if (bGetSubItems)// if we have to get them
    {
        // get first child
        hTreeItem=TreeView_GetChild(this->pSelectWindow->WindowHandle,hTreeItem);
        if (hTreeItem)
        {
            // write children header
            if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
                CTextFile::WriteText(this->hFile, _T("</TD><TD><TABLE BORDER=\"1\">"));
            else if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_TXT)
                CTextFile::WriteText(this->hFile, _T("\r\n"));

            // for each child
            while(hTreeItem)
            {
                // write child header
                if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
                    CTextFile::WriteText(this->hFile, _T("<TR><TD>"));

                // write child content
                this->SaveTreeViewItemContent(hTreeItem,ItemDepth+1,bExpandedOnly);

                // write child footer
                if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
                    CTextFile::WriteText(this->hFile, _T("</TD></TR>"));

                // get next child
                hTreeItem=TreeView_GetNextSibling(this->pSelectWindow->WindowHandle,hTreeItem);
            }

            // write children footer
            if (this->SavingType==REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML)
                CTextFile::WriteText(this->hFile,_T("</TABLE>"));
        }
    }

    // item footer
    this->WriteTreeViewItemFooter();

    return TRUE;
}

DWORD CRemoteCtrlContentSaver::GetTargetProcessId()
{
	if (!this->pSelectWindow)
		return (DWORD)-1;

	return this->pSelectWindow->WindowProcessID;
}

HWND CRemoteCtrlContentSaver::GetTargetHandle()
{
	if (!this->pSelectWindow)
		return 0;

	return this->pSelectWindow->WindowHandle;
}