/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


/////////////////////////////////
// include
/////////////////////////////////
#include <windows.h>
#include <commctrl.h>
#pragma warning (push)
#pragma warning(disable : 4005)// for '_stprintf' : macro redefinition in tchar.h
#include <TCHAR.h>
#pragma warning (pop)
#include "resource.h"
#include "about.h"


// Enable Visual Style
#if defined _AMD64
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='amd64' publicKeyToken='6595b64144ccf1df' language='*'\"")
#elif defined _WIN64
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='ia64' publicKeyToken='6595b64144ccf1df' language='*'\"")
#else
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='x86' publicKeyToken='6595b64144ccf1df' language='*'\"")
#endif

#include "remotectrlcontentsaver.h"
#include "../tools/GUI/SelectWindow/selectwindow.h"
#include "../tools/GUI/Dialog/DialogHelper.h"
#include "../tools/String/MultipleElementsParsing.h"
#include "../Tools/Clipboard/Clipboard.h"
#include "../Tools/File/StdFileOperations.h"
#include "../tools/Privilege/privilege.h"
#include "../tools/Process/ProcessHelper/ProcessHelper.h"

/////////////////////////////////
// global vars
/////////////////////////////////
CSelectWindow* pSelectWindow=NULL;
CRemoteCtrlContentSaver* pRemoteCtrlContentSaver=NULL;
enum CRemoteCtrlContentSaver::REMOTECTRLCONTENTSAVER_SUPPORTED_CTRL_TYPES SelectedControlType=CRemoteCtrlContentSaver::SUPPORTED_CTRL_UNSUPPORTED;
HWND mhWndDialog=NULL;
HINSTANCE mhInstance;
BOOL bCommandeLineMode=FALSE;
TCHAR pszCommandLineOutPutFile[MAX_PATH]=_T("");
BOOL bCommandeLineSelectedItemsOnly=FALSE;
BOOL bCommandeLineExpandedItemsOnly=FALSE;
BOOL bCommandeLineOnlySomeColumns=FALSE;
BOOL bCommandLineCrossCompatibility=FALSE;
BOOL bCommandeLineShowPassword=FALSE;
BOOL bCommandeLineCopyPasswordToClipboard=FALSE;
BOOL bCommandeLineSavePassword=FALSE;
TCHAR pszCommandLineColumnsNumber[MAX_PATH]=_T("");

/////////////////////////////////
// function declare
/////////////////////////////////
LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
void OnMouseDown(LPARAM lParam);
void OnMouseUp(LPARAM lParam);
void OnMouseMove(LPARAM lParam);
void Exit(INT_PTR nExitCode);
BOOL Save();
void About();
void Init();
BOOL SaveDialog(TCHAR* pszFile,DWORD pszFileMaxSize);
BOOL WriteContent(TCHAR* pszFileName,TCHAR* pszContent);
BOOL ParseCommandLine(OUT HWND* pTargetHwnd);
void CopySelectedTreeViewItemToClipBoard();

#define COMMAND_LINE_WINDOW_HANDLE L"Hwnd="
#define COMMAND_LINE_OUTPUT_FILE_ARG L"File="
#define COMMAND_LINE_SELECTED_ITEMS_ONLY L"OnlySelected"
#define COMMAND_LINE_EXPANDED_ITEMS_ONLY L"OnlyExpanded"
#define COMMAND_LINE_ONLY_COLUMNS L"OnlyColumns="
#define COMMAND_LINE_SHOW_PASSWORD L"ShowPwd"
#define COMMAND_LINE_COPY_PASSWORD_TO_CLIPBOARD L"CopyPwd"
#define COMMAND_LINE_SAVE_PASSWORD L"SavePwd"
#define COMMAND_LINE_CROSS_COMPATIBILITY L"CrossCompatibility"


#define COMMAND_LINE_SELECTED_ITEMS_ONLY_T TEXT("OnlySelected")
#define COMMAND_LINE_EXPANDED_ITEMS_ONLY_T TEXT("OnlyExpanded")
#define COMMAND_LINE_ONLY_COLUMNS_T TEXT("OnlyColumns=")
#define COMMAND_LINE_SHOW_PASSWORD_T TEXT("ShowPwd")
#define COMMAND_LINE_COPY_PASSWORD_TO_CLIPBOARD_T TEXT("CopyPwd")
#define COMMAND_LINE_SAVE_PASSWORD_T TEXT("SavePwd")

#define CROSS_COMPATIBILITY_EXE_NAME64 _T("ControlContentSaver64.exe")
#define CROSS_COMPATIBILITY_EXE_NAME32 _T("ControlContentSaver.exe")

//-----------------------------------------------------------------------------
// Name: ParseCommandLine
// Object: parse command line
//          fill globals var 
//            TCHAR pszCommandLineOutPutFile[MAX_PATH]=_T("");
//            BOOL bCommandeLineSelectedItemsOnly=FALSE;
//            BOOL bCommandeLineOnlySomeColumns=FALSE;
//            TCHAR pszCommandLineColumnsNumber[MAX_PATH]=_T("");
// Parameters : 
//     in : LPSTR lpCmdLine : command line
//     out: HWND* pTargetHwnd handle of the window
// Return : TRUE on success
//-----------------------------------------------------------------------------
BOOL ParseCommandLine(OUT HWND* pTargetHwnd)
{
    WCHAR* pData;
    LPWSTR* lppwargv;
    int argc=0;
    int cnt;
    int iScanfRes=0;

    *pTargetHwnd=NULL;

    // use CommandLineToArgvW
    lppwargv=CommandLineToArgvW(GetCommandLineW(),&argc);
    // if no params
    if (argc<=1)
    {
        LocalFree(lppwargv);
        return FALSE;
    }

#ifdef _DEBUG
    MessageBox(0,_T("Ready to debug command line parsing"),_T("Information"),MB_OK|MB_ICONINFORMATION);
#endif

    for(cnt=1;cnt<argc;cnt++)
    {
        if (wcsnicmp(lppwargv[cnt],COMMAND_LINE_WINDOW_HANDLE,wcslen(COMMAND_LINE_WINDOW_HANDLE))==0)
        {
            pData=lppwargv[cnt]+wcslen(COMMAND_LINE_WINDOW_HANDLE);
            
            if(wcsnicmp(pData,L"0x",2)==0)
                iScanfRes=swscanf(pData+2,L"%x",pTargetHwnd);
            else
                iScanfRes=swscanf(pData,L"%u",pTargetHwnd);

            if (iScanfRes!=1)
                break;
        }
        else if (wcsnicmp(lppwargv[cnt],COMMAND_LINE_OUTPUT_FILE_ARG,wcslen(COMMAND_LINE_OUTPUT_FILE_ARG))==0)
        {
            pData=lppwargv[cnt]+wcslen(COMMAND_LINE_OUTPUT_FILE_ARG);
#if (defined(UNICODE)||defined(_UNICODE))
            wcscpy(pszCommandLineOutPutFile,pData);
#else
            LPSTR p;
            CAnsiUnicodeConvert::UnicodeToAnsi(pData,&p);
            strcpy(pszCommandLineOutPutFile,p);
            free(p);
#endif
        }
        else if (wcsicmp(lppwargv[cnt],COMMAND_LINE_SELECTED_ITEMS_ONLY)==0)
        {
            bCommandeLineSelectedItemsOnly=TRUE;
        }
        else if(wcsicmp(lppwargv[cnt],COMMAND_LINE_EXPANDED_ITEMS_ONLY)==0)
        {
            bCommandeLineExpandedItemsOnly=TRUE;
        }
        else if (wcsnicmp(lppwargv[cnt],COMMAND_LINE_ONLY_COLUMNS,wcslen(COMMAND_LINE_ONLY_COLUMNS))==0)
        {
            bCommandeLineOnlySomeColumns=TRUE;
            pData=lppwargv[cnt]+wcslen(COMMAND_LINE_ONLY_COLUMNS);
#if (defined(UNICODE)||defined(_UNICODE))
            wcscpy(pszCommandLineColumnsNumber,pData);
#else
            LPSTR p;
            CAnsiUnicodeConvert::UnicodeToAnsi(pData,&p);
            strcpy(pszCommandLineColumnsNumber,p);
            free(p);
#endif
        }
		else if(wcsicmp(lppwargv[cnt],COMMAND_LINE_CROSS_COMPATIBILITY)==0)
		{
			bCommandLineCrossCompatibility=TRUE;
		}
		else if(wcsicmp(lppwargv[cnt],COMMAND_LINE_SHOW_PASSWORD)==0)
		{
			bCommandeLineShowPassword=TRUE;
		}
		else if(wcsicmp(lppwargv[cnt],COMMAND_LINE_COPY_PASSWORD_TO_CLIPBOARD)==0)
		{
			bCommandeLineCopyPasswordToClipboard=TRUE;
		}
		else if(wcsicmp(lppwargv[cnt],COMMAND_LINE_SAVE_PASSWORD)==0)
		{
			bCommandeLineSavePassword=TRUE;
		}
		else
		{
			WCHAR Msg[MAX_PATH];
			_snwprintf(Msg,MAX_PATH,L"Unknown command line option %s",lppwargv[cnt]);
			MessageBoxW(NULL,Msg,L"Launcher Error",MB_OK|MB_ICONERROR|MB_TOPMOST);
			break;
		}

    }
    LocalFree(lppwargv);

    if (*pTargetHwnd==0) // if no target window
        return FALSE;

    // if not cross compatibility command line
    if (!bCommandLineCrossCompatibility)
    {
        // if saving file name is not set
        if (*pszCommandLineOutPutFile==0)
        {
            // allowed only for password showing and clipboard copy
            if ( (bCommandeLineCopyPasswordToClipboard || bCommandeLineShowPassword)            
                  && (!bCommandeLineSavePassword)
                )
                return FALSE;
        }
    }
    return TRUE;
}

/////////////////////////////////
// main
/////////////////////////////////
int WINAPI WinMain(HINSTANCE hInstance,
    HINSTANCE hPrevInstance,
    LPSTR lpCmdLine,
    int nCmdShow
)
{
    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(nCmdShow);

    // check if we are in command line
    bCommandeLineMode=(*lpCmdLine!=0);

    mhInstance=hInstance;

    // try to get debug privileges to get other users launched application content
    CPrivilege Privilege(FALSE);
    Privilege.SetPrivilege(SE_DEBUG_NAME,TRUE);

    // create CSelectWindow object 
    pSelectWindow=new CSelectWindow();

    if (bCommandeLineMode)
    {
        HWND TargetHwnd=NULL;

        if (!ParseCommandLine(&TargetHwnd))
            return -1;

        pRemoteCtrlContentSaver=new CRemoteCtrlContentSaver(bCommandLineCrossCompatibility);

        if(!pSelectWindow->GetHandleInfo(TargetHwnd))
            return -1;

        SelectedControlType=pRemoteCtrlContentSaver->UpdateControl(pSelectWindow);

		// in case of cross compatibility call, init XP syle because there will be user interaction dialogs
		if (bCommandLineCrossCompatibility)
		{
			// to enable XP visual style
			InitCommonControls();
		}

        if (!Save())
            return -1;
    }
    else
    {

        // to enable XP visual style
        InitCommonControls();
        // show main window
        DialogBox(mhInstance, (LPCTSTR)IDD_DIALOG_MAIN, NULL, (DLGPROC)WndProc);

    }

    // delete objects
    if (pSelectWindow)
        delete pSelectWindow;

    if (pRemoteCtrlContentSaver)
        delete pRemoteCtrlContentSaver;

    return 0;
}

// main window proc
LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{

    switch (uMsg)
    {
    case WM_INITDIALOG:
        mhWndDialog=hWnd;
        Init();

        break;
    case WM_CLOSE:
        Exit(LOWORD(wParam));
        break;
    case WM_LBUTTONDOWN:
        OnMouseDown(lParam);
        break;

    case WM_LBUTTONUP:
    case WM_KILLFOCUS:
        OnMouseUp(lParam);
        break;

    case WM_MOUSEMOVE:
        OnMouseMove(lParam);
        break;

    case WM_COMMAND:
        switch (LOWORD(wParam))
        {
        case IDC_BUTTON_EXIT:
            Exit(LOWORD(wParam));
            break;
        case IDC_BUTTON_SAVE:
            Save(); 
            break;
        case IDC_BUTTON_ABOUT:
            About();
            break;
        case IDC_RADIO_TREEVIEW_SELECTED:
            if (SelectedControlType==CRemoteCtrlContentSaver::SUPPORTED_CTRL_TREEVIEW)
                CopySelectedTreeViewItemToClipBoard();
            break;
        }
        break;
    }

    return FALSE;
}
/////////////////////////
// Init
/////////////////////////
void Init()
{
    // set application icon
    CDialogHelper::SetIcon(mhWndDialog,IDI_ICON_APP);

    // init select window
    pSelectWindow->Initialize(mhInstance,mhWndDialog,IDC_STATIC_SELECT_TARGET,
        IDB_BITMAP_BLANK,IDB_BITMAP_CROSS,IDC_CURSOR);

    // check password radio show and copy
    SendDlgItemMessage(mhWndDialog,IDC_RADIO_PASSWORD_SHOW_AND_COPY,(UINT)BM_SETCHECK,BST_CHECKED,0);

    // check treeview radio Expanded only
    SendDlgItemMessage(mhWndDialog,IDC_RADIO_TREEVIEW_EXPANDED_ONLY,(UINT)BM_SETCHECK,BST_CHECKED,0);

    //  init pRemoteCtrlContentSaver
    pRemoteCtrlContentSaver=new CRemoteCtrlContentSaver(mhWndDialog);
}

/////////////////////////
// OnMouseDown, OnMouseUp, OnMouseMove just used for selecting control
/////////////////////////
void OnMouseDown(LPARAM lParam)
{
    pSelectWindow->MouseDown(lParam);
}
void OnMouseUp(LPARAM lParam)
{
    UNREFERENCED_PARAMETER(lParam);

    if (pSelectWindow->MouseUp())
    {
        // user interface update
        switch(SelectedControlType)
        {
        case CRemoteCtrlContentSaver::SUPPORTED_CTRL_UNSUPPORTED:
        case CRemoteCtrlContentSaver::SUPPORTED_CTRL_EDIT: 
        case CRemoteCtrlContentSaver::SUPPORTED_CTRL_EDIT_PASSWORD:
            // disable the save button
            EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_SAVE),FALSE);
            // do special operation for these controls
            Save();
            break;

        case CRemoteCtrlContentSaver::SUPPORTED_CTRL_TREEVIEW:
            // enable the save button
            EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_SAVE),TRUE);
            // if only selected item -> auto copy it to clipboard
            if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_TREEVIEW_SELECTED)==BST_CHECKED)
                CopySelectedTreeViewItemToClipBoard();
            break;

        default:
            // enable the save button
            EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_SAVE),TRUE);
            break;
        }
    }
}

void OnMouseMove(LPARAM lParam)
{
    BOOL bWindowChanged=FALSE;
    
    pSelectWindow->MouseMove(lParam,&bWindowChanged);
    if (bWindowChanged)
    {
        // get control type
        SelectedControlType=pRemoteCtrlContentSaver->UpdateControl(pSelectWindow);

        // put handle value in IDC_EDIT_HANDLE
        TCHAR pcMsg[32];
        _stprintf(pcMsg,_T("0x%.8X"),pSelectWindow->WindowHandle);
        SetDlgItemText(mhWndDialog,IDC_EDIT_HANDLE,pcMsg);

        // show class name
        //if (SelectedControlType==CRemoteCtrlContentSaver::SUPPORTED_CTRL_EDIT_PASSWORD)
        //    // if password field show password instead of edit
        //    SetDlgItemText(mhWndDialog,IDC_EDIT_CLASS_NAME,"Password");
        //else
            SetDlgItemText(mhWndDialog,IDC_EDIT_CLASS_NAME,pSelectWindow->WindowClassName);
    }
}

/////////////////////////
// Generic text content saving
/////////////////////////
BOOL WriteContent(TCHAR* pszFileName,TCHAR* pszContent)
{
    HANDLE hFile;
    TCHAR* pszXMLLike;

    if(!CTextFile::CreateTextFile(pszFileName,&hFile))
        return FALSE;

    // get file type
    CRemoteCtrlContentSaver::REMOTECTRLCONTENTSAVER_SAVING_TYPES SavingType;
    if (CStdFileOperations::DoesExtensionMatch(pszFileName,_T("xml")))
        SavingType=CRemoteCtrlContentSaver::REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML;
    else if (CStdFileOperations::DoesExtensionMatch(pszFileName,_T("html")))
        SavingType=CRemoteCtrlContentSaver::REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML;
    else
        SavingType=CRemoteCtrlContentSaver::REMOTECTRLCONTENTSAVER_SAVING_TYPE_TXT;

    
    switch(SavingType)
    {
    case CRemoteCtrlContentSaver::REMOTECTRLCONTENTSAVER_SAVING_TYPE_XML:
        // write header
        CTextFile::WriteText(hFile,_T("<TEXT>"));
        // write content
        pszXMLLike=new TCHAR[_tcslen(pszContent)*REPLACED_CHAR_MAX_INCREASE];
        CXMLLikeConvert::Convert(pszContent,pszXMLLike,TRUE);
        CTextFile::WriteText(hFile,pszXMLLike);
        delete pszXMLLike;
        // write footer
        CTextFile::WriteText(hFile,_T("</TEXT>"));
        break;
    case CRemoteCtrlContentSaver::REMOTECTRLCONTENTSAVER_SAVING_TYPE_HTML:
        // write header
        CTextFile::WriteText(hFile,_T("<HTML><BODY>"));
        // write content
        pszXMLLike=new TCHAR[_tcslen(pszContent)*REPLACED_CHAR_MAX_INCREASE];
        CXMLLikeConvert::Convert(pszContent,pszXMLLike,TRUE);
        CTextFile::WriteText(hFile,pszXMLLike);
        delete pszXMLLike;
        // write footer
        CTextFile::WriteText(hFile,_T("</BODY></HTML>"));
        break;
    case CRemoteCtrlContentSaver::REMOTECTRLCONTENTSAVER_SAVING_TYPE_TXT:
    default:
        CTextFile::WriteText(hFile,pszContent);
        break;

    }
    CloseHandle(hFile);

    return TRUE;
}
/////////////////////////
// Save handler
/////////////////////////
BOOL Save()
{
    TCHAR pszColumnsNumber[MAX_PATH];
    *pszColumnsNumber=0;
    SIZE_T NbSelectedColomns=0;
    SIZE_T Cnt;
    SIZE_T* pColumnsNumber=NULL;
    BOOL bSelectedItemsOnly=FALSE;
    BOOL bExpandedItemsOnly=FALSE;
    TCHAR* pszContent=NULL;

    TCHAR pszText[MAX_PATH];
    BOOL bRet=TRUE;

	// check cross compatibility checking : 
	// as structs contains pointers, WIN32 and WIN64 user32 structures are different,
	// and has there's no templates for structs (thanks m$) the easiest way is to call
	// the ControlContentSaver with same architecture type as targeted process
	DWORD TargetProcessId;
	BOOL b32bitTarget = TRUE;
	BOOL bCheckTarget;
	TargetProcessId = pRemoteCtrlContentSaver->GetTargetProcessId();
	if (TargetProcessId == (DWORD)-1)
	{
		// error getting target process id
		bCheckTarget = FALSE;
	}
	else
	{
		bCheckTarget = CProcessHelper::Is32bitsProcess(TargetProcessId,&b32bitTarget);
	}

	TCHAR* CrossExeName=CROSS_COMPATIBILITY_EXE_NAME32;
	BOOL bNeedCrossCompatibility = FALSE;
	if (bCheckTarget)
	{
#ifdef _WIN64
		if(b32bitTarget)
		{
			// need to call 32 bit version of control content saver
			bNeedCrossCompatibility = TRUE;
			CrossExeName = CROSS_COMPATIBILITY_EXE_NAME32;
		}
#else
		if(!b32bitTarget)
		{
			// need to call 64 bit version of control content saver
			bNeedCrossCompatibility = TRUE;
			CrossExeName = CROSS_COMPATIBILITY_EXE_NAME64;
		}
#endif
	}

	if (bNeedCrossCompatibility)
	{
		TCHAR CrossCompatibilityCommandLine[2*MAX_PATH];
		TCHAR AppPath[MAX_PATH];
		STARTUPINFO si={0};
		PROCESS_INFORMATION pi={0};

		CStdFileOperations::GetAppPath(AppPath,MAX_PATH);
		_tcscat(AppPath,CrossExeName);

		HWND hWnd = pRemoteCtrlContentSaver->GetTargetHandle();



		_sntprintf(CrossCompatibilityCommandLine,_countof(CrossCompatibilityCommandLine),
					_T("\"%s\" %s %s0x%x"),
					AppPath,
					COMMAND_LINE_CROSS_COMPATIBILITY,
					COMMAND_LINE_WINDOW_HANDLE,
					hWnd
					);

		// get user interface informations and forge command lines parameter according to these informations
		BOOL bShowPassword=FALSE;
		BOOL bCopyPasswordToClipboard=FALSE;
		BOOL bSelectedItemsOnly = FALSE;
		switch (SelectedControlType)
		{
		case CRemoteCtrlContentSaver::SUPPORTED_CTRL_EDIT_PASSWORD:
			if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_PASSWORD_SHOW)==BST_CHECKED)
				bShowPassword=TRUE;

			if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_PASSWORD_COPY_ONLY)==BST_CHECKED)
				bCopyPasswordToClipboard=TRUE;

			if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_PASSWORD_SHOW_AND_COPY)==BST_CHECKED)
			{
				bCopyPasswordToClipboard=TRUE;
				bShowPassword=TRUE;
			}
			if (bShowPassword)
			{
				_tcscat(CrossCompatibilityCommandLine,_T(" "));
				_tcscat(CrossCompatibilityCommandLine,COMMAND_LINE_SHOW_PASSWORD_T);
			}
			if (bCopyPasswordToClipboard)
			{
				_tcscat(CrossCompatibilityCommandLine,_T(" "));
				_tcscat(CrossCompatibilityCommandLine,COMMAND_LINE_COPY_PASSWORD_TO_CLIPBOARD_T);
			}
			break;
		case CRemoteCtrlContentSaver::SUPPORTED_CTRL_TREEVIEW:
			if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_TREEVIEW_SELECTED)==BST_CHECKED)
				bSelectedItemsOnly=TRUE;
			if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_TREEVIEW_EXPANDED_ONLY)==BST_CHECKED)
			{
				_tcscat(CrossCompatibilityCommandLine,_T(" "));
				_tcscat(CrossCompatibilityCommandLine,COMMAND_LINE_EXPANDED_ITEMS_ONLY_T);
			}
			break;

		case CRemoteCtrlContentSaver::SUPPORTED_CTRL_LISTVIEW:
			if (IsDlgButtonChecked(mhWndDialog,IDC_CHECK_SAVE_ONLY_SELECTED_LISTVIEW)==BST_CHECKED)
				bSelectedItemsOnly=TRUE;
			break;

		case CRemoteCtrlContentSaver::SUPPORTED_CTRL_LISTBOX:
			if (IsDlgButtonChecked(mhWndDialog,IDC_CHECK_SAVE_ONLY_SELECTED_LISTBOX)==BST_CHECKED)
				bSelectedItemsOnly=TRUE;
			break;
		}

		if (bSelectedItemsOnly)
		{
			_tcscat(CrossCompatibilityCommandLine,_T(" "));
			_tcscat(CrossCompatibilityCommandLine,COMMAND_LINE_SELECTED_ITEMS_ONLY_T);
		}

		if ( (SelectedControlType==CRemoteCtrlContentSaver::SUPPORTED_CTRL_HEADERS)
			||(SelectedControlType==CRemoteCtrlContentSaver::SUPPORTED_CTRL_LISTVIEW)
			)
		{
			if (IsDlgButtonChecked(mhWndDialog,IDC_CHECK_SAVE_ONLY_COLUMNS)==BST_CHECKED)
			{
				// get selected columns number
				if (GetDlgItemText(mhWndDialog,IDC_EDIT_NUM_COLUMNS_TO_SAVE,pszColumnsNumber,MAX_PATH))
				{
					_tcscat(CrossCompatibilityCommandLine,_T(" "));
					_tcscat(CrossCompatibilityCommandLine,COMMAND_LINE_ONLY_COLUMNS_T);
					_tcscat(CrossCompatibilityCommandLine,pszColumnsNumber);
				}
			}
		}

		si.cb=sizeof(si);
		if (!::CreateProcess(0,CrossCompatibilityCommandLine,0,0,0,0,0,0,&si,&pi))
		{
			if(!bCommandeLineMode)
			{
				TCHAR Msg[2*MAX_PATH];
				_sntprintf(Msg,_countof(Msg),
							_T("Error launching %s\r\n")
							_T("for 32/64 bit cross compatibility"),
							AppPath);
				::MessageBox(mhWndDialog,Msg,_T("Error"),MB_OK|MB_ICONERROR);
			}
			return FALSE;
		}
		
		::CloseHandle(pi.hProcess);
		::CloseHandle(pi.hThread);
		return TRUE;
	}

    // get saving file name
    TCHAR pszFile[MAX_PATH]=_T("");

    if (bCommandeLineMode)
    {
        _tcsncpy(pszFile,pszCommandLineOutPutFile,MAX_PATH-1);
        pszFile[MAX_PATH-1]=0;
    }

    switch (SelectedControlType)
    {
    // until SUPPORTED_CTRL_UNSUPPORTED will send WM_GETTEXT message
    case CRemoteCtrlContentSaver::SUPPORTED_CTRL_UNSUPPORTED:
    case CRemoteCtrlContentSaver::SUPPORTED_CTRL_EDIT: 
        // get only text content
        if (pRemoteCtrlContentSaver->GetTextContent(&pszContent))
        {
            if (bCommandeLineMode && !bCommandLineCrossCompatibility)
            {
                if (!WriteContent(pszFile,pszContent))
                    bRet=FALSE;
            }
            else
            {
                // put in dialog
                _tcscpy(pszText,_T("The following text has been copied to clipboard :\r\n"));
                _tcsncat(pszText,pszContent,MAX_PATH-_tcslen(pszText)-4);
                if (_tcslen(pszText)>=MAX_PATH-4)
                    _tcscat(pszText,_T("..."));
                else
                    pszText[MAX_PATH-4]=0;

                // copy to clipboard
                if (CClipboard::CopyToClipboard(mhWndDialog,pszContent))
                    MessageBox(mhWndDialog,pszText,_T("Information"),MB_OK|MB_ICONINFORMATION);
            }
        }
        else
            bRet=FALSE;
        break;
    case CRemoteCtrlContentSaver::SUPPORTED_CTRL_EDIT_PASSWORD:
		{
			BOOL bSavePassword=FALSE;
			BOOL bShowPassword=FALSE;
			BOOL bCopyPasswordToClipboard=FALSE;
			if (bCommandeLineMode)
			{
				bShowPassword=bCommandeLineShowPassword;
				bCopyPasswordToClipboard=bCommandeLineCopyPasswordToClipboard;
				bSavePassword=bCommandeLineSavePassword;

				// backward compatibility for previous version where bSavePassword was defaulted
				if ( (!bShowPassword) && (!bCopyPasswordToClipboard) )
					bSavePassword = TRUE;
			}
			else
			{
				// get user interface informations
				if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_PASSWORD_SHOW)==BST_CHECKED)
					bShowPassword=TRUE;

				if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_PASSWORD_COPY_ONLY)==BST_CHECKED)
					bCopyPasswordToClipboard=TRUE;

				if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_PASSWORD_SHOW_AND_COPY)==BST_CHECKED)
				{
					bCopyPasswordToClipboard=TRUE;
					bShowPassword=TRUE;
				}
			}

			if (bSavePassword)
			{
				if ( (!bCommandeLineMode) || bCommandLineCrossCompatibility)
				{
					if (!SaveDialog(pszFile,MAX_PATH))
						return TRUE;
				}

				if(pRemoteCtrlContentSaver->GetPasswordContent(&pszContent))
				{
					if (!WriteContent(pszFile,pszContent))
						bRet=FALSE;
				}
				else
					bRet=FALSE;

				if ( (!bCommandeLineMode) || bCommandLineCrossCompatibility)
				{
					if (bRet)
						MessageBox(mhWndDialog,_T("Password successfully saved"),_T("Information"),MB_OK|MB_ICONINFORMATION);
					else
						MessageBox(mhWndDialog,_T("Error getting password"),_T("Error"),MB_OK|MB_ICONERROR);
				}
			}

			// if password should be show
			if (bShowPassword)
				pRemoteCtrlContentSaver->ShowPasswordContent();
			// if copy to clipboard
			if (bCopyPasswordToClipboard)
			{
				if (pRemoteCtrlContentSaver->GetPasswordContent(&pszContent))
				{
					// copy to clipboard
					bRet = CClipboard::CopyToClipboard(mhWndDialog,pszContent);

					if ( (!bCommandeLineMode) || bCommandLineCrossCompatibility)
					{
						if (bRet)
							MessageBox(mhWndDialog,_T("Password successfully paste into clipboard"),_T("Information"),MB_OK|MB_ICONINFORMATION);
						else
							MessageBox(mhWndDialog,_T("Error pasting password into clipboard"),_T("Error"),MB_OK|MB_ICONERROR);
					}
				}
				else
				{
					bRet=FALSE;
					if ( (!bCommandeLineMode) || bCommandLineCrossCompatibility)
						MessageBox(mhWndDialog,_T("Error getting password"),_T("Error"),MB_OK|MB_ICONERROR);
				}
			}
		}
        break;
    case CRemoteCtrlContentSaver::SUPPORTED_CTRL_COMBOBOX:
        if ( (!bCommandeLineMode) || bCommandLineCrossCompatibility)
        {
            if (!SaveDialog(pszFile,MAX_PATH))
                return TRUE;
        }
        bRet=pRemoteCtrlContentSaver->SaveComboBoxContent(pszFile);
        break;
    case CRemoteCtrlContentSaver::SUPPORTED_CTRL_COMBOBOXEX32:
        if ( (!bCommandeLineMode) || bCommandLineCrossCompatibility)
        {
            if (!SaveDialog(pszFile,MAX_PATH))
                return TRUE;
        }
        bRet=pRemoteCtrlContentSaver->SaveComboBoxEx32Content(pszFile);
        break;
    case CRemoteCtrlContentSaver::SUPPORTED_CTRL_LISTBOX:
        if (bCommandeLineMode)
        {
            bSelectedItemsOnly=bCommandeLineSelectedItemsOnly;
        }
        else
        {
            if (IsDlgButtonChecked(mhWndDialog,IDC_CHECK_SAVE_ONLY_SELECTED_LISTBOX)==BST_CHECKED)
                bSelectedItemsOnly=TRUE;
        }

		if ( (!bCommandeLineMode) || bCommandLineCrossCompatibility)
		{
			if (!SaveDialog(pszFile,MAX_PATH))
				return TRUE;
		}

        bRet=pRemoteCtrlContentSaver->SaveListBoxContent(pszFile,bSelectedItemsOnly);
        break;
    case CRemoteCtrlContentSaver::SUPPORTED_CTRL_HEADERS:
        if (bCommandeLineMode)
        {
            if (bCommandeLineOnlySomeColumns)
            {
                pColumnsNumber=CMultipleElementsParsing::ParseSIZE_T(pszCommandLineColumnsNumber,&NbSelectedColomns);
                // translate 1 base index (user entry) to 0 base index (subitem index value)
                for(Cnt=0;Cnt<NbSelectedColomns;Cnt++)
                    pColumnsNumber[Cnt]--;
            }
        }
        else
        {
            if (IsDlgButtonChecked(mhWndDialog,IDC_CHECK_SAVE_ONLY_COLUMNS)==BST_CHECKED)
            {
                // get selected columns number
                if (GetDlgItemText(mhWndDialog,IDC_EDIT_NUM_COLUMNS_TO_SAVE,pszColumnsNumber,MAX_PATH))
                {
                    pColumnsNumber=CMultipleElementsParsing::ParseSIZE_T(pszColumnsNumber,&NbSelectedColomns);
                    // translate 1 base index (user entry) to 0 base index (subitem index value)
                    for(Cnt=0;Cnt<NbSelectedColomns;Cnt++)
                        pColumnsNumber[Cnt]--;
                }
            }

        }

		if ( (!bCommandeLineMode) || bCommandLineCrossCompatibility)
		{
			if (!SaveDialog(pszFile,MAX_PATH))
				return TRUE;
		}

        bRet=pRemoteCtrlContentSaver->SaveHeaderContent(pszFile,pColumnsNumber,NbSelectedColomns);
        break;
    case CRemoteCtrlContentSaver::SUPPORTED_CTRL_LISTVIEW:
        if (bCommandeLineMode)
        {
            bSelectedItemsOnly=bCommandeLineSelectedItemsOnly;
            if (bCommandeLineOnlySomeColumns)
            {
                pColumnsNumber=CMultipleElementsParsing::ParseSIZE_T(pszCommandLineColumnsNumber,&NbSelectedColomns);
                // translate 1 base index (user entry) to 0 base index (subitem index value)
                for(Cnt=0;Cnt<NbSelectedColomns;Cnt++)
                    pColumnsNumber[Cnt]--;
            }
        }
        else
        {
            if (IsDlgButtonChecked(mhWndDialog,IDC_CHECK_SAVE_ONLY_SELECTED_LISTVIEW)==BST_CHECKED)
                bSelectedItemsOnly=TRUE;
            if (IsDlgButtonChecked(mhWndDialog,IDC_CHECK_SAVE_ONLY_COLUMNS)==BST_CHECKED)
            {
                // get selected columns number
                if (GetDlgItemText(mhWndDialog,IDC_EDIT_NUM_COLUMNS_TO_SAVE,pszColumnsNumber,MAX_PATH))
                {
                    pColumnsNumber=CMultipleElementsParsing::ParseSIZE_T(pszColumnsNumber,&NbSelectedColomns);
                    // translate 1 base index (user entry) to 0 base index (subitem index value)
                    for(Cnt=0;Cnt<NbSelectedColomns;Cnt++)
                        pColumnsNumber[Cnt]--;
                }
            }
        }

		if ( (!bCommandeLineMode) || bCommandLineCrossCompatibility)
		{
			if (!SaveDialog(pszFile,MAX_PATH))
				return TRUE;
		}

        bRet=pRemoteCtrlContentSaver->SaveListViewContent(pszFile,bSelectedItemsOnly,pColumnsNumber,NbSelectedColomns);
        break;

    case CRemoteCtrlContentSaver::SUPPORTED_CTRL_TREEVIEW:
        if (bCommandeLineMode)
        {
            bExpandedItemsOnly=bCommandeLineExpandedItemsOnly;
            if (!bExpandedItemsOnly)
                bSelectedItemsOnly=bCommandeLineSelectedItemsOnly;
        }
        else
        {
            if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_TREEVIEW_EXPANDED_ONLY)==BST_CHECKED)
                bExpandedItemsOnly=TRUE;
            if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_TREEVIEW_SELECTED)==BST_CHECKED)
                bSelectedItemsOnly=TRUE;
        }

		if ( (!bCommandeLineMode) || bCommandLineCrossCompatibility)
		{
			if (!SaveDialog(pszFile,MAX_PATH))
				return TRUE;
		}

        bRet=pRemoteCtrlContentSaver->SaveTreeViewContent(pszFile,bExpandedItemsOnly,bSelectedItemsOnly);
        break;

    }

    // free memory
    if (pColumnsNumber)
        delete[] pColumnsNumber;

    if (pszContent)
        free(pszContent);

    return bRet;
}

void CopySelectedTreeViewItemToClipBoard()
{
    TCHAR* psz;
    if(pRemoteCtrlContentSaver->GetTreeViewSelectedContent(&psz))
    {
        CClipboard::CopyToClipboard(mhWndDialog,psz);
        TCHAR* pszTmp=new TCHAR[_tcslen(psz)+1+100];
        _tcscpy(pszTmp,_T("The following text has been copied to clipboard :\r\n"));
        _tcscat(pszTmp,psz);
        MessageBox(mhWndDialog,pszTmp,_T("Information"),MB_OK|MB_ICONINFORMATION);
        delete[] pszTmp;
        delete[] psz;
    }
}

//-----------------------------------------------------------------------------
// Name: SaveDialog
// Object: This function shows an interface to user for choosing file
// Parameters :
//     in  : DWORD pszFileMaxSize
//     out : 
//     inout : TCHAR* pszFile
//     return : TRUE on save, FALSE on cancel
//-----------------------------------------------------------------------------
BOOL SaveDialog(TCHAR* pszFile,DWORD pszFileMaxSize)
{
    // open dialog
    OPENFILENAME ofn;
    memset(&ofn,0,sizeof (OPENFILENAME));
    ofn.lStructSize=sizeof (OPENFILENAME);
    ofn.hwndOwner=NULL;
    ofn.hInstance=NULL;
    ofn.lpstrFilter=_T("html, xml, txt\0*.html;*.xml;*.txt\0html\0*.html\0xml\0*.xml\0txt\0*.txt\0");
    ofn.nFilterIndex = 1;
    ofn.Flags=OFN_EXPLORER|OFN_NOREADONLYRETURN|OFN_OVERWRITEPROMPT;
    ofn.lpstrDefExt=_T("html");
    ofn.lpstrFile=pszFile;
    ofn.nMaxFile=pszFileMaxSize;

    return GetSaveFileName(&ofn);
}

/////////////////////////
// Exit handler
/////////////////////////
void Exit(INT_PTR nExitCode)
{
    EndDialog(mhWndDialog,nExitCode);
}

void About()
{
    CAbout::Show(mhInstance, mhWndDialog,NULL);
}