/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: class helper for displaying a messagebox containing the win32 error message 
//         after an API failure
//         As this class is used in many other tools class, and if you don't want to 
//         report error to the user, just define the keyword "TOOLS_NO_MESSAGEBOX"
//         in preprocessor options
//-----------------------------------------------------------------------------


#include "APIError.h"
#ifndef TOOLS_NO_MESSAGEBOX
    #include <stdio.h>
#endif
#ifndef _countof
	#define _countof(array) (sizeof(array)/sizeof(array[0]))
#endif

HWND CAPIError::hParentWindow = NULL;
CAPIError::pfRtlNtStatusToDosError CAPIError::pRtlNtStatusToDosError = NULL;

void CAPIError::SetParentWindowHandle(HWND ParentWindow)
{
    CAPIError::hParentWindow = ParentWindow;
}
ULONG CAPIError::NtStatusToDosError(NTSTATUS Status)
{
    if (!CAPIError::pRtlNtStatusToDosError)
    {
        HMODULE hModule = ::GetModuleHandle(_T("ntdll.dll"));
        CAPIError::pRtlNtStatusToDosError = (pfRtlNtStatusToDosError)::GetProcAddress(hModule,"RtlNtStatusToDosError");
        if (!CAPIError::pRtlNtStatusToDosError)
            return ERROR_INVALID_FUNCTION;
    }
    return CAPIError::pRtlNtStatusToDosError(Status);
}

//-----------------------------------------------------------------------------
// Name: ShowLastError
// Object: show last windows api error
// Parameters :
//     in  :
//     out :
//     return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
BOOL CAPIError::ShowLastError(TCHAR* szPrefix,HWND ParentWindow)
{
#if (defined(TOOLS_NO_MESSAGEBOX))
    UNREFERENCED_PARAMETER(szPrefix);
    UNREFERENCED_PARAMETER(ParentWindow);
    return TRUE;
#else
    TCHAR pcError[MAX_PATH];
    SIZE_T MsgSize;
    TCHAR* pszMsg;
    DWORD dwLastError=GetLastError();
    DWORD dw=FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
                           NULL,
                           dwLastError,
                           GetUserDefaultLangID(),//GetSystemDefaultLangID()
                           pcError,
                           MAX_PATH-1,
                           NULL);
    
    MsgSize = (_tcslen(szPrefix)+MAX_PATH+6);// +3 for \r\n and \0
    pszMsg=(TCHAR*)_alloca(MsgSize*sizeof(TCHAR)); 
    _tcscpy(pszMsg,szPrefix);

    //If the function succeeds, the return value is the number of TCHARs stored in the output buffer,
    //  excluding the terminating null character.
    //If the function fails, the return value is zero
    if(dw==0)
    {
        // FormatMessage failed
        _sntprintf(pszMsg,MsgSize,_T("%s Error 0x%08X\r\n"),szPrefix,dwLastError);
    }
    else
    {
        // FormatMessage is ok add error description to pszMsg
        _tcscat(pszMsg,_T("\r\n"));
        _tcscat(pszMsg,pcError);
    }

    HWND LocalParentWindow;

    // if a specific parent window is defined, use it
    if (ParentWindow)
        LocalParentWindow = ParentWindow;
    else // else, use the default one
        LocalParentWindow = CAPIError::hParentWindow;

    return MessageBox(LocalParentWindow,pszMsg,_T("Error"),MB_OK|MB_ICONERROR);

#endif
}

//-----------------------------------------------------------------------------
// Name: ShowLastError
// Object: show last windows api error
// Parameters :
//     in  :
//     out :
//     return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
BOOL CAPIError::ShowLastError(HWND ParentWindow)
{
#if (defined(TOOLS_NO_MESSAGEBOX))
    UNREFERENCED_PARAMETER(ParentWindow);
    return TRUE;
#else
    TCHAR pcError[MAX_PATH];
    DWORD dwLastError=GetLastError();
    DWORD dw=FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
                           NULL,
                           dwLastError,
                           GetUserDefaultLangID(),//GetSystemDefaultLangID()
                           pcError,
                           MAX_PATH-1,
                           NULL);
    
    
    //If the function succeeds, the return value is the number of TCHARs stored in the output buffer,
    //  excluding the terminating null character.
    //If the function fails, the return value is zero
    if(dw==0)
    {
        // FormatMessage failed
        _sntprintf(pcError,_countof(pcError),_T("Error 0x%08X"),dwLastError);
    }

    HWND LocalParentWindow;

    // if a specific parent window is defined, use it
    if (ParentWindow)
        LocalParentWindow = ParentWindow;
    else // else, use the default one
        LocalParentWindow = CAPIError::hParentWindow;

    return MessageBox(LocalParentWindow,pcError,_T("Error"),MB_OK|MB_ICONERROR);
#endif
}
//-----------------------------------------------------------------------------
// Name: ShowError
// Object: show windows api error (avoid to remove last error value)
// Parameters :
//     in  : DWORD dwLastError
//     out :
//     return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
BOOL CAPIError::ShowError(DWORD dwLastError,HWND ParentWindow)
{
#if (defined(TOOLS_NO_MESSAGEBOX))
    UNREFERENCED_PARAMETER(dwLastError);
    UNREFERENCED_PARAMETER(ParentWindow);
    return TRUE;
#else
    TCHAR pcError[MAX_PATH];
    DWORD dw=FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
                           NULL,
                           dwLastError,
                           GetUserDefaultLangID(),//GetSystemDefaultLangID()
                           pcError,
                           MAX_PATH-1,
                           NULL);
    
    
    //If the function succeeds, the return value is the number of TCHARs stored in the output buffer,
    //  excluding the terminating null character.
    //If the function fails, the return value is zero
    if(dw==0)
    {
        // FormatMessage failed
        _sntprintf(pcError,_countof(pcError),_T("Error 0x%08X"),dwLastError);
    }

    HWND LocalParentWindow;

    // if a specific parent window is defined, use it
    if (ParentWindow)
        LocalParentWindow = ParentWindow;
    else // else, use the default one
        LocalParentWindow = CAPIError::hParentWindow;

    return MessageBox(LocalParentWindow,pcError,_T("Error"),MB_OK|MB_ICONERROR);
#endif
}

//-----------------------------------------------------------------------------
// Name: ShowError
// Object: show windows api error
// Parameters :
//     in  :
//     out :
//     return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
BOOL CAPIError::ShowError(TCHAR* szPrefix,DWORD dwError,HWND ParentWindow)
{
#if (defined(TOOLS_NO_MESSAGEBOX))
    UNREFERENCED_PARAMETER(szPrefix);
    UNREFERENCED_PARAMETER(dwError);
    UNREFERENCED_PARAMETER(ParentWindow);
    return TRUE;
#else
    TCHAR pcError[MAX_PATH];
    TCHAR* pszMsg;
    SIZE_T MsgSize;
    DWORD dw=FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
                           NULL,
                           dwError,
                           GetUserDefaultLangID(),//GetSystemDefaultLangID()
                           pcError,
                           MAX_PATH-1,
                           NULL);
    
    MsgSize = _tcslen(szPrefix)+MAX_PATH+6; // +3 for \r\n and \0
    pszMsg=(TCHAR*)_alloca(MsgSize*sizeof(TCHAR)); 
    _tcscpy(pszMsg,szPrefix);

    //If the function succeeds, the return value is the number of TCHARs stored in the output buffer,
    //  excluding the terminating null character.
    //If the function fails, the return value is zero
    if(dw==0)
    {
        // FormatMessage failed
        _sntprintf(pszMsg,MsgSize,_T("%s Error 0x%08X\r\n"),szPrefix,dwError);
    }
    else
    {
        // FormatMessage is ok add error description to pszMsg
        _tcscat(pszMsg,_T("\r\n"));
        _tcscat(pszMsg,pcError);
    }

    HWND LocalParentWindow;

    // if a specific parent window is defined, use it
    if (ParentWindow)
        LocalParentWindow = ParentWindow;
    else // else, use the default one
        LocalParentWindow = CAPIError::hParentWindow;

    return MessageBox(LocalParentWindow,pszMsg,_T("Error"),MB_OK|MB_ICONERROR);

#endif
}


//-----------------------------------------------------------------------------
// Name: ShowError
// Object: show windows api error
// Parameters :
//     in  :
//     out :
//     return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
BOOL CAPIError::ShowError(TCHAR* szPrefix,DWORD dwError,TCHAR* ModuleName,HWND ParentWindow)
{
#if (defined(TOOLS_NO_MESSAGEBOX))
    UNREFERENCED_PARAMETER(szPrefix);
    UNREFERENCED_PARAMETER(dwError);
    UNREFERENCED_PARAMETER(ModuleName);
    UNREFERENCED_PARAMETER(ParentWindow);
    return TRUE;
#else
    HMODULE hModule = ::LoadLibrary(ModuleName);
    if (!hModule)
        return CAPIError::ShowError(szPrefix,dwError,hParentWindow);

    TCHAR pcError[MAX_PATH];
    TCHAR* pszMsg;
    SIZE_T MsgSize;
    DWORD dw=FormatMessage(FORMAT_MESSAGE_FROM_HMODULE,
                            hModule,
                            dwError,
                            GetUserDefaultLangID(),//GetSystemDefaultLangID()
                            pcError,
                            MAX_PATH-1,
                            NULL);

    MsgSize = _tcslen(szPrefix)+MAX_PATH+6; // +3 for \r\n and \0
    pszMsg=(TCHAR*)_alloca(MsgSize*sizeof(TCHAR)); 
    _tcscpy(pszMsg,szPrefix);

    //If the function succeeds, the return value is the number of TCHARs stored in the output buffer,
    //  excluding the terminating null character.
    //If the function fails, the return value is zero
    if(dw==0)
    {
        // FormatMessage failed
        _sntprintf(pszMsg,MsgSize,_T("%s Error 0x%08X\r\n"),szPrefix,dwError);
    }
    else
    {
        // FormatMessage is ok add error description to pszMsg
        _tcscat(pszMsg,_T("\r\n"));
        _tcscat(pszMsg,pcError);
    }

    HWND LocalParentWindow;

    // if a specific parent window is defined, use it
    if (ParentWindow)
        LocalParentWindow = ParentWindow;
    else // else, use the default one
        LocalParentWindow = CAPIError::hParentWindow;

    ::FreeLibrary(hModule);
    return MessageBox(LocalParentWindow,pszMsg,_T("Error"),MB_OK|MB_ICONERROR);

#endif
}

//-----------------------------------------------------------------------------
// Name: ShowError
// Object: show windows api error
// Parameters :
//     in  :
//     out :
//     return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
BOOL CAPIError::ShowError(TCHAR* szPrefix,DWORD dwError,HMODULE hModule,HWND ParentWindow)
{
#if (defined(TOOLS_NO_MESSAGEBOX))
    UNREFERENCED_PARAMETER(szPrefix);
    UNREFERENCED_PARAMETER(dwError);
    UNREFERENCED_PARAMETER(hModule);
    UNREFERENCED_PARAMETER(ParentWindow);
    return TRUE;
#else
    TCHAR pcError[MAX_PATH];
    TCHAR* pszMsg;
    SIZE_T MsgSize;
    DWORD dw=FormatMessage(FORMAT_MESSAGE_FROM_HMODULE,
        hModule,
        dwError,
        GetUserDefaultLangID(),//GetSystemDefaultLangID()
        pcError,
        MAX_PATH-1,
        NULL);

    MsgSize = _tcslen(szPrefix)+MAX_PATH+6; // +3 for \r\n and \0
    pszMsg=(TCHAR*)_alloca(MsgSize*sizeof(TCHAR)); 
    _tcscpy(pszMsg,szPrefix);

    //If the function succeeds, the return value is the number of TCHARs stored in the output buffer,
    //  excluding the terminating null character.
    //If the function fails, the return value is zero
    if(dw==0)
    {
        // FormatMessage failed
        _sntprintf(pszMsg,MsgSize,_T("%s Error 0x%08X\r\n"),szPrefix,dwError);
    }
    else
    {
        // FormatMessage is ok add error description to pszMsg
        _tcscat(pszMsg,_T("\r\n"));
        _tcscat(pszMsg,pcError);
    }

    HWND LocalParentWindow;

    // if a specific parent window is defined, use it
    if (ParentWindow)
        LocalParentWindow = ParentWindow;
    else // else, use the default one
        LocalParentWindow = CAPIError::hParentWindow;

    return MessageBox(LocalParentWindow,pszMsg,_T("Error"),MB_OK|MB_ICONERROR);
#endif
}